#include "MustuxLib/MustuxDebugger.hh"
#include <string>
#include <iostream>
#include <cstdio>
#include <linux/soundcard.h>
#include "Audio.hh"
#include "PrafFile.hh"
#include "WavFile.hh"


// FOR LATER ::#include "GlobalSettings.hh"

Audio::Audio()
        {
        }

Audio::~Audio()
        {
        }


int Audio::build(string aFileName)
        {
        PENTER;
        filename = aFileName;
        // TODO : replace the dirty if below by something like "toLowerCase"
        if (
            (filename.find("praf")!=string::npos) ||
            (filename.find("Praf")!=string::npos) ||
            (filename.find("PRAF")!=string::npos)
            )

                audioType = PRAF;
        // TODO : replace the dirty if below by something like "toLowerCase"
        if (
            (filename.find("mp3")!=string::npos) ||
            (filename.find("Mp3")!=string::npos) ||
            (filename.find("MP3")!=string::npos)
            )
                audioType = MPEG3;
        else
        // TODO : replace the dirty if below by something like "toLowerCase"
        if (
            (filename.find("ogg")!=string::npos) ||
            (filename.find("Ogg")!=string::npos) ||
            (filename.find("OGG")!=string::npos)
            )
                audioType = OGGVORBIS;
        else
        // TODO : replace the dirty if below by something like "toLowerCase"
        if (
            (filename.find("wav")!=string::npos) ||
            (filename.find("Wav")!=string::npos) ||
            (filename.find("WAV")!=string::npos)
            )
                audioType = WAV;


        switch (audioType)
                {
                case PRAF :
                        {
                        PrafFile *pf;
                        pf = new PrafFile();
                        if (pf->open(filename)<0)
                                {
                                PERROR("File not Found");
                                delete pf;
                                PEXIT;
                                return -1;
                                }
                        file = pf->get_file();
                        std::rewind(file);
                        pf->read_header();
                        headerSize = pf->get_header_size();
                        audioSize = pf->get_audio_size();
                        channels = pf->get_channels();
                        rate = pf->get_rate();
                        bitDepth = pf->get_bit_depth();
                        fileSize = audioSize + headerSize;
                        bytesPerSec = rate * channels * (bitDepth / 8);
                        blockSize = channels * (bitDepth / 8);
                        sampleSize = blockSize / channels;
                        totalBlocks = audioSize / blockSize;
                        peak = new Peak();
                        peak->build(this);
                        rewind();
                        delete pf;
                        break;
                        }
                case WAV:
                        {
                        WavFile *wf;
                        wf = new WavFile();
                        if (wf->open(filename)<0)
                                {
                                PERROR("File not Found");
                                delete wf;
                                PEXIT;
                                return -1;
                                }
                        file = wf->get_file();
                        std::rewind(file);
                        wf->read_header();
                        headerSize = wf->get_header_size();
                        audioSize = wf->get_audio_size();
                        channels = wf->get_channels();
                        rate = wf->get_rate();
                        bitDepth = wf->get_bit_depth();
                        fileSize = audioSize + headerSize;
                        bytesPerSec = rate * channels * (bitDepth / 8);
                        blockSize = channels * (bitDepth / 8);
                        sampleSize = blockSize / channels;
                        totalBlocks = audioSize / blockSize;
                        peak = new Peak();
                        peak->build(this);
                        rewind();
                        delete wf;
                        break;
                        }
                case OGGVORBIS:
                case MPEG3:
                default:
                        {
                        PERROR ("Format not supported inside Protux. Try Genpraf to convert it to PRAF format.");
                        }
                }
        PEXIT;
        return 1;
        }



int Audio::create(string aFileName, int aChannels, int aRate, int aSize,
                        string aProjectLabel, string aGroupLabel,
                        string aHardwareLabel, string aDescription)
        {
        PENTER;
        PMESG("Creating audio : "  << aChannels << " channels," <<
                                        aRate << " Hz, " << aSize << " bits" << endl <<
                                        "\t\tProject : " << aProjectLabel << endl <<
                                        "\t\tGroup : " << aGroupLabel << endl <<
                                        "\t\tHardware : " << aHardwareLabel << endl <<
                                        "\t\tDesc : " << aDescription);
        PrafFile *pf =  new PrafFile();
        if (pf->create(aFileName)<0)
                {
                delete pf;
                PERROR("Cannot create new file");
                return -1;
                }
        pf->write_header(aChannels, aRate, aSize, 0,
                         aProjectLabel, aGroupLabel,
                         aHardwareLabel, aDescription);

        headerSize = pf->get_header_size();
        audioSize = pf->get_audio_size();
        fileSize = audioSize + headerSize;
        channels = pf->get_channels();
        rate = pf->get_rate();
        bitDepth = pf->get_bit_depth();
        bytesPerSec = rate * channels * (bitDepth / 8);
        blockSize = channels * (bitDepth / 8);
        totalBlocks = audioSize / blockSize;
        delete pf;
        PEXIT;
        return 1;
        }


int Audio::unbuild()
        {
        PENTER;
        if (file!=NULL)
                if (fclose(file)<0)
                        PERROR("Cannot close file" );
        PEXIT;
        return 1;
        }


void Audio::rewind()
        {
        fseek(file, headerSize, SEEK_SET);
        }


void Audio::goto_end()
        {
        fseek(file, 0, SEEK_END);
        }


void Audio::locate( long long blockpos)
        {
        fseek(file, headerSize + (blockpos * blockSize), SEEK_SET);
        }


long long Audio::get_location()
        {
        long long p = (ftell(file) - headerSize) / blockSize;
        return p;
        }


FILE* Audio::get_file()
        {
        return file;
        }


long long Audio::get_file_size()
        {
        return fileSize;
        }

long long Audio::get_audio_size()
        {
        return audioSize;
        }

int Audio::get_block_size()
        {
        return blockSize;
        }

int Audio::get_header_size()
        {
        return headerSize;
        }


long long Audio::get_total_blocks()
        {
        return totalBlocks;
        }


int Audio::get_rate()
        {
        return rate;
        }


int Audio::get_bit_depth()
        {
        return bitDepth;
        }


int Audio::get_channels()
        {
        return channels;
        }



string Audio::get_filename()
        {
        return filename;
        }


void Audio::set_filename(string fn)
        {
        filename = fn;
        }


long long Audio::get_last_block() // ???
        {
        long lastfilebyte = ftell(file);
        long lastaudiobyte = lastfilebyte - headerSize;
        long lastblock = lastaudiobyte / blockSize;
        return lastblock;
        }





        // eof


