#ifndef AUDIO_HH
#define AUDIO_HH

#include <string>
#include <cstdio>

class Peak;

class Audio
    {
    public :
        static const int PRAF=1;
        static const int OGGVORBIS=2;
        static const int WAV=3;
        static const int MPEG3=4;
        Peak* peak;

        Audio();
        ~Audio();
        int build(string aFileName);
        int create(string aFileName, int aChannels, int aRate, int aSize,
                string aProjectLabel, string aGroupLabel,
                string aHardwareLabel, string aDescription);
        int unbuild();
        void locate(long long blockpos);
        long long get_location();
        void rewind();
        void goto_end();
        long long get_current_block();
        long long get_file_size();
        long long get_audio_size();
        long long get_total_blocks();
        long long get_last_block();
        int get_block_size();
        int get_rate();
        int get_bit_depth();
        int get_channels();
        int get_header_size();
        string get_filename();
        void set_filename(string fn);
        FILE* get_file();
        int fix_size();

    private:
        int audioType;
        FILE* file;
        string filename;
        int file_status;
        int channels;
        int rate;
        int bitDepth;
        int blockSize;
        int sampleSize; // in bytes
        long long fileSize;
        long long headerSize;
        long long audioSize;
        long long bytesPerSec;
        long long totalBlocks;
        long long filePos;
        int fix_header();


    };

#include "Peak.hh"

#endif


