#include "AudioClip.hh"
#include "MustuxLib/MustuxDebugger.hh"
#include "MustuxLib/MustuxColorManager.hh"


AudioClip::AudioClip(Track* aParent, Audio *anAudioSource, long long aSourceFirstBlock, long long aLenght, long long aTrackInsertBlock)
        {
        PENTERCONS;
        parentTrack = aParent;
        audioSource = anAudioSource;
        drawPane=parentTrack->parentMTA->get_draw_pane();
        storePane=parentTrack->parentMTA->get_store_pane();
        blockSize = audioSource->get_block_size();
        channels = audioSource->get_channels();
        maxSample = (int) pow(2,audioSource->get_bit_depth());
        bytesPerSample = blockSize/channels;
        parentMTA = parentTrack->parentMTA;
        sourceFirstBlock= aSourceFirstBlock;
        lenght = aLenght;
        sourceLastBlock = sourceFirstBlock + lenght;
        trackFirstBlock = aTrackInsertBlock;
        trackLastBlock = trackFirstBlock + lenght;
        PEXITCONS;
        }


AudioClip::~AudioClip()
        {
        PENTERDES;
        PEXITDES;
        }



void AudioClip::set_mta_container( MultiTrackArea* mta)
        {
        parentMTA= mta;
        }


        
Audio* AudioClip::get_audio_source()
        {
        return audioSource;
        }


int AudioClip::draw()
        {
        PENTER;
        PMESG("Drawing clip " << this);

        int baseY  = parentTrack->baseY;
        int height = parentTrack->height - 2;
        int trackWidth = parentTrack->width;
        static GdkGC* colorWhite = MustuxColorManager::get_color(MustuxColorManager::WHITE);
        static GdkGC* colorBlack = MustuxColorManager::get_color(MustuxColorManager::BLACK);
        static GdkGC* colorPeak = MustuxColorManager::get_color(MustuxColorManager::PLAYING_PEAK_COLOR);

        int vzoom = parentMTA->get_vzoom();
        int hzoom = parentMTA->get_hzoom();

        int rootX = parentMTA->block_to_xpos(trackFirstBlock);
        int clipXWidth = parentMTA->block_to_xpos(sourceLastBlock)-parentMTA->block_to_xpos(sourceFirstBlock);

        long long firstBlock = sourceFirstBlock;

        if (rootX < 0)
                {
                firstBlock += parentMTA->xpos_to_block(-1*rootX);
                rootX = 0;
                }

        if (clipXWidth > (trackWidth - rootX))
                clipXWidth = trackWidth - rootX;

        // PMESG("trackFirstBlock = " << trackFirstBlock << " screen pos for this =  " << rootX);

        gdk_draw_rectangle (storePane, MustuxColorManager::get_color(MustuxColorManager::CLIP_BACKGROUND_COLOR), true,
                                rootX, baseY, clipXWidth , height);

        for (int i=0; i<3; i++)
                {
                gdk_draw_line(storePane,colorWhite, rootX + i , baseY , rootX + i , baseY + height );
                gdk_draw_line(storePane,colorWhite, rootX     , baseY + i , rootX + clipXWidth , baseY + i );
                gdk_draw_line(storePane,colorBlack, rootX + i , baseY + height - i , rootX + clipXWidth , baseY + height - i );
                gdk_draw_line(storePane,colorBlack, rootX + clipXWidth - i , baseY + height , rootX + clipXWidth - i , baseY + i );
                }
        gdk_draw_rectangle (storePane, colorBlack, false, rootX, baseY , clipXWidth , height);

        int peaksToFill = clipXWidth;
PMARK(7);
        char* peakBuffer = new char[ peaksToFill * blockSize ];
        int peaksFilled = audioSource->peak->fill_buffer(peakBuffer, firstBlock, peaksToFill, hzoom);
PMARK(8);

        short sample;
        const int chHeight = height / channels;
        const float f = (float) chHeight / maxSample;
        const int yheight = height / channels;
        const int ycenter = yheight / 2;
        int y = 0;
        int yc = 0;

        for (int k=0; k<channels; k++)
                {
                yc = ycenter+yheight*k;
                char* p = peakBuffer;
                int lasty = yc;
                for (int x=0; x<peaksFilled; x++)
                        {
                        sample = *(short*)(p+sizeof(short)*k);
                        y = yc - (int) (f * (float) sample * vzoom) ;
                        gdk_draw_line (storePane, colorPeak, rootX + x - 1, baseY + lasty , rootX + x ,  baseY + y);
                        lasty=y;
                        p+=blockSize;
                        }
                }
        delete peakBuffer;

        PEXIT;
        return 1;
        }


void AudioClip::set_track_first_block(long long newTrackFirstBlock)
        {
        trackFirstBlock=newTrackFirstBlock;
        trackLastBlock=trackFirstBlock + lenght;
        }



int AudioClip::play()
        {
        return 1;
        }

int AudioClip::split()
        {
        return 1;
        }

void AudioClip::mute()
        {
        isMuted=true;
        }

void AudioClip::unmute()
        {
        isMuted=false;
        }

// eof


