// THIS CLASS MUST BE FULLY REVISED

#include <iostream>
#include <cstdio>
#include <cstdlib>
#include "Dsp.hh"
#include "MustuxLib/MustuxLib.hh"
// FOR LATER ::#include "GlobalSettings.hh"


Dsp::Dsp()
    {
    PENTERCONS;
    Id = 0;
    audioDevice = "/dev/dsp";// FOR LATER ::GlobalSettings::get_audio_device();
    PMESG("Creating Dsp controller for audio device " << audioDevice);
    PEXITCONS;
    }



Dsp::~Dsp()
    {
    PENTERDES;
    PEXITDES;
    }



int Dsp::open_for_play()
    {
    int driver=OSS; // TEMPORARIO, olhe a linha abaixo
    switch (driver)// FOR LATER ::GlobalSettings::get_driver())
        {
        case OSS:
            {
            Id = std::open(audioDevice.c_str(), O_WRONLY);
            if (Id < 0)
                {
                PERROR("Device is busy");
                }
            return Id;
            }
        case ALSA:
            {
            PMESG("Alsa support not implemented yet." );
            return -1;
            }
        }
    return -1;
    }



int Dsp::open_for_rec()
        {
        int driver=OSS;
        switch (driver)//GlobalSettings::get_driver())
                {
                case OSS:
                        {
                        Id = std::open(audioDevice.c_str(), O_RDONLY);
                        if (Id < 0)
                                PERROR("Cannot open audio device " << audioDevice);
                        return Id;
                        }
                case ALSA:
                        {
                        PMESG("Alsa support not implemented yet." );
                        return -1;
                        }
                }
        return -1;
        }



int Dsp::set(int channels, int rate, int size)
        {
        int driver=OSS;
        switch (driver)//GlobalSettings::get_driver())
                {
                case OSS:
                        {
                        static int arg , r;
                        arg = channels;
                        r = ioctl(Id, SOUND_PCM_WRITE_CHANNELS, &arg);
                        arg = size;
                        r = ioctl(Id, SOUND_PCM_WRITE_BITS, &arg);
                        arg = rate;
                        r = ioctl(Id, SOUND_PCM_WRITE_RATE, &arg);
                        return 1;
                        }
                case ALSA:
                        {
                        PMESG("Alsa support not implemented yet." );
                        return -1;
                        }
                }
        return -1;
        }



int Dsp::set_format(int requested_format)
        {
        int driver=OSS;
        switch (driver)//GlobalSettings::get_driver())
                {
                case OSS:
                        {
                        int format;
                        format = requested_format;
                        if (ioctl (Id, SNDCTL_DSP_SETFMT, &format) == -1)
                                {
                                PERROR("Cannot set sample format");
                                return -1;
                                }
                        if (format != requested_format)
                                {
                                PERROR("Dsp::set_format() : sample format not supported");
                                return -1;
                                }
                        return 1;
                        }
                case ALSA:
                        {
                        PMESG("Alsa support not implemented yet." );
                        return -1;
                        }
                }
        return -1;
        }



int Dsp::close()
        {
        int driver=OSS;
        switch (driver)//GlobalSettings::get_driver())
                {
                case OSS:
                        {
                        if (std::close(Id) == -1)
                                {
                                PERROR("Cannot close audio device " << audioDevice);
                                return -1;
                                }
                        else
                                return 1;
                        }
                case ALSA:
                        {
                        PMESG("Alsa support not implemented yet." );
                        return -1;
                        }
                }
        return -1;
        }




int Dsp::get_best_fragment_size()
        {
        int driver=OSS;
        switch (driver)//GlobalSettings::get_driver())

                {
                case OSS:
                        {
                        int best_frag_size_in_bytes;
                        if (ioctl(Id, SNDCTL_DSP_GETBLKSIZE, &best_frag_size_in_bytes) == -1)
                                {
                                PERROR("Cannot set optimal fragment size");
                                return -1;
                                }
                        return best_frag_size_in_bytes;
                        }
                case ALSA:
                        {
                        PMESG("Alsa support not implemented yet." );
                        return -1;
                        }
                }
        return -1;
        }




int Dsp::play_fragment(void* fragment,  int fragment_size)
        {
        if (std::write( Id , fragment , fragment_size ) == -1)
                {
                PERROR("Dsp::play_fragment() : cannot write to DSP");
                return -1;
                }
        return 1;
        }




int Dsp::rec_fragment(void* fragment,  int fragment_size)
        {
        if (std::read( Id , fragment, fragment_size ) == -1)
                {
                PERROR("Cannot read fragment from audio device " << audioDevice);
                return -1;
                }
        return 1;
        }






string Dsp::get_supported_formats()
        {
        PENTER;
        int mask;
        int driver=OSS;
        switch (driver)//GlobalSettings::get_driver())

            {
            case OSS:
                {
                if (ioctl(Id, SNDCTL_DSP_GETFMTS, &mask) == -1)
                        PERROR("Cannot retrieve supported sample formats (is audio device open?)");
                break;
                }
            case ALSA:
                {
                PMESG("Alsa support not implemented yet.");
                break;
                }
            }
        if (mask & AFMT_MPEG)
            PMESG("MPEG support OK");
        if (mask & AFMT_MU_LAW)
            PMESG("AFMT_MU_LAW support OK");
        if (mask & AFMT_A_LAW)
            PMESG("AFMT_A_LAW support OK");
        if (mask & AFMT_IMA_ADPCM)
            PMESG("AFMT_IMA_ADPCM support OK");
        if (mask & AFMT_U8)
            PMESG("AFMT_U8 support OK");
        if (mask & AFMT_S16_LE)
            PMESG("AFMT_S16_LE support OK");
        if (mask & AFMT_S16_BE)
            PMESG("AFMT_S16_BE support OK");
        if (mask & AFMT_S16_NE)
            PMESG("AFMT_S16_NE support OK");
        if (mask & AFMT_U16_LE)
            PMESG("AFMT_U16_LE support OK");
        if (mask & AFMT_U16_BE)
            PMESG("AFMT_U16_BE support OK");
        PEXIT;
        return "" ;
        }


// eof



