#include <iostream>
#include <string>
#include "MustuxLib/MustuxLib.hh"
#include "Interface.hh"
#include "FileSelector.hh"
#include "PrafSettingsWindow.hh"


Interface::Interface():MustuxInterface()
        {
        PENTERCONS;
        PEXITCONS;
        }


Interface::~Interface()
        {
        PENTERDES;
        PEXITDES;
        }


int Interface::build()
        {
        PENTER;
        MustuxInterface::build();
        set_app_name("PROTUX");
        set_app_version("0.12");// FOR LATER SettingsManager::get_property("VERSION_INFO");
        set_app_desc("The Free Professional Audio Tools for Linux");
        //set_title(get_app_name());

        projectMenu = add_menu("Project");
        miNewProject = add_menu_item(projectMenu,"New Project");
        miOpenProject = add_menu_item(projectMenu,"Open Project");
        miSaveProject = add_menu_item(projectMenu,"Save Project");
        miCloseProject = add_menu_item(projectMenu,"Close Project");

        audioMenu = add_menu("Audio");
        miImportAudio = add_menu_item(audioMenu,"Import");
        miExportAudio = add_menu_item(audioMenu,"Export");

        renderMenu = add_menu("Render");
        miRenderTrack = add_menu_item(renderMenu,"Render Track");
        miRenderProject = add_menu_item(renderMenu,"Render Project");

        settingsMenu = add_menu("Settings");
        miSettings = add_menu_item(settingsMenu,"Global Settings");

        mustacheMenu = add_menu("Mustache");
        miNewMusic = add_menu_item(mustacheMenu, "New Music");
        miOpenMusic = add_menu_item(mustacheMenu, "Open Music");

        helpMenu = add_menu("Help");
        miHelp = add_menu_item(helpMenu,"Protux Help");
        miAbout = add_menu_item(helpMenu,"About Protux");

        // THIS IS THE PLACE TO INSERT THE PROTUX LOGO WIDGET

        //add_separator();

        operationLcd = new MustuxLcd(lcdsContainer);
        operationLcd->build(65,100,MustuxComponent::NOT_RESIZEABLE);

        lcdsContainer->add_separator();

        filterInfoLcd = new MustuxLcd(lcdsContainer);
        filterInfoLcd->build(240,100,MustuxComponent::NOT_RESIZEABLE);

        PEXIT;
        return 1;
        }




int Interface::init()
        {
        PENTER;
        MustuxInterface::init();
        MustuxColorManager::build(window);
        MustuxColorManager::init();
        operationLcd->init();
        filterInfoLcd->init();
        ML_CONNECT_CALLBACK(miNewProject,"activate",on_new_project);
        ML_CONNECT_CALLBACK(miOpenProject,"activate",on_open_project);
        ML_CONNECT_CALLBACK(miSaveProject,"activate",on_save_project);
        ML_CONNECT_CALLBACK(miCloseProject,"activate",on_close_project);
        ML_CONNECT_CALLBACK(miImportAudio,"activate",on_import_audio);
        ML_CONNECT_CALLBACK(miExportAudio,"activate",on_export_audio);
        ML_CONNECT_CALLBACK(miRenderTrack,"activate",on_render_track);
        ML_CONNECT_CALLBACK(miRenderProject,"activate",on_render_project);
        ML_CONNECT_CALLBACK(miSettings,"activate",on_settings);
        ML_CONNECT_CALLBACK(miNewMusic,"activate",on_new_music);
        ML_CONNECT_CALLBACK(miOpenMusic,"activate",on_open_music);
        ML_CONNECT_CALLBACK(miHelp, "activate", on_show_help);
        ML_CONNECT_CALLBACK(miAbout, "activate", on_show_about);
        currProject=-1;
        show_splash_screen();
        set_welcome_msg("WELCOME TO PROTUX 0.12.0");
        say_welcome();

        PEXIT;
        return 1;
        }


int Interface::finalize()
        {
        PENTER;
        MustuxInterface::finalize();
        PEXIT;
        return 1;
        }


int Interface::unbuild()
        {
        PENTER;
        delete jmb;
        delete filterInfoLcd;
        delete operationLcd;
        MustuxInterface::unbuild();
        PEXIT;
        return 1;
        }


int Interface::process_action(string actionName/*, int grabbedVal*/) // its terrible ! needs an optimization!!!
        {
        PENTER;
        // The next line is causing a trash dump...
        //PMESG("Processing " << actionName << " action");
        if (actionName == "QUIT")
                {
                MustuxInterface::on_exit();
                }
        if (actionName == "ABOUT")
                {
                }
        if (currProject>=0)
                {
                if (actionName == "TOUCH")
                        {
                        multiTrackArea[currProject]->touch();
                        }
                else if (actionName == "SPLIT")
                        {
                        multiTrackArea[currProject]->split();
                        }
                else if (actionName == "HORIZONTAL ZOOM IN")
                        {
                        multiTrackArea[currProject]->hzoom_in();
                        }
                else if (actionName == "HORIZONTAL ZOOM OUT")
                        {
                        multiTrackArea[currProject]->hzoom_out();
                        }
                else if (actionName == "SCROLL RIGHT")
                        {
                        multiTrackArea[currProject]->scroll_right();
                        }
                else if (actionName == "SCROLL LEFT")
                        {
                        multiTrackArea[currProject]->scroll_left();
                        }
                else if (actionName == "PLAY/STOP")
                        {
                        multiTrackArea[currProject]->toggle_play_stop();
                        }
                else if (actionName == "JOG-SLIDE")
                        {
                        multiTrackArea[currProject]->jog_slide();
                        }
                else if (actionName == "FILE STUFF")
                        {
                        on_import_audio();
                        }
                else if (actionName == "RENDER ALL")
                        {
                        multiTrackArea[currProject]->render_all();
                        }

                }
        PEXIT;
        return 1;
        }


int Interface::new_project(int ntracks)
        {
        PENTER;
        currProject = projectsContainer->add_project();
        projectsContainer->set_label(currProject,"untitled.pp");
        GtkWidget* targetBox = projectsContainer->get_current_project_container();
        multiTrackPanel[currProject] = new MultiTrackPanel(targetBox);
        multiTrackPanel[currProject]->build(ntracks);
        multiTrackPanel[currProject]->init();

        multiTrackArea[currProject] = new MultiTrackArea(targetBox);
        multiTrackArea[currProject]->build(ntracks, 640,360);
        multiTrackArea[currProject]->init();

        audioSourcesWindow = new AudioSourcesWindow();
        audioSourcesWindow->build(300,200);
        audioSourcesWindow->init();
        PEXIT;
        return 1;
        }


static int _hide_splash_screen(gpointer obj)
        {
        Interface* o = (Interface*) obj;
        o->hide_splash_screen();
        return 1;
        }



void Interface::show_splash_screen()
        {
        PENTER;
        aboutWindow = new MustuxAboutWindow();
        aboutWindow->build("aboutProtux.jpg");
        aboutWindow->init();
        aboutTag = gtk_timeout_add( 3000, _hide_splash_screen, this );
        PEXIT;
        }


void Interface::hide_splash_screen()
        {
        PENTER;
        delete aboutWindow;
        gtk_timeout_remove(aboutTag);
        PEXIT;
        }


// callbacks

ML_DEFINE_CALLBACK(on_new_project, Interface)
        {
        new_project(4);
        }


ML_DEFINE_CALLBACK(on_open_project, Interface)
        {
        }

ML_DEFINE_CALLBACK(on_save_project, Interface)
        {
        }

ML_DEFINE_CALLBACK(on_close_project, Interface)
        {
        }

ML_DEFINE_CALLBACK(on_import_audio, Interface)
        {
        PENTER;
        jmb->suspend();
        FileSelector* fs = new FileSelector();
        string filename = fs->get();
        if (filename!="")
                {
                if (currProject<0)
                        {
                        new_project(4);
                        }
                multiTrackArea[currProject]->fill_track(filename);
                }
        delete fs;
        jmb->activate();
        PEXIT;
        }


ML_DEFINE_CALLBACK(on_export_audio, Interface)
{
}

ML_DEFINE_CALLBACK(on_render_track, Interface)
{
}

ML_DEFINE_CALLBACK(on_render_project, Interface)
{
}

ML_DEFINE_CALLBACK(on_new_music, Interface)
{
}

ML_DEFINE_CALLBACK(on_open_music, Interface)
{
}

ML_DEFINE_CALLBACK(on_settings, Interface)
{
}


ML_DEFINE_CALLBACK(on_show_help, Interface)
{
}


ML_DEFINE_CALLBACK(on_show_about, Interface)
{
}

// EOF

