#ifndef INTERFACE_HH
#define INTERFACE_HH

#include <gtk/gtk.h>
#include <string>
#include "MustuxLib/MustuxLib.hh"


class MustuxComponent;
class MustuxLcd;
class MustuxWindow;

class MultiTrackArea;
class MultiTrackPanel;
class AudioSourcesWindow;
class MustuxAboutWindow;

class Interface : public MustuxInterface
    {
    public :
        static const int MAX_PROJECTS_OPEN = 100;

        Interface();
        ~Interface();
        int build();
        int init();
        int finalize();
        int unbuild();
        int process_action(string actionName/*, int grabbedVal*/);
        void show_splash_screen();
        void hide_splash_screen();


    private :
        int currProject;
        MustuxLcd *operationLcd;
        MustuxLcd *filterInfoLcd;
        MustuxLcd *jmbLcd;
        MultiTrackArea *multiTrackArea[MAX_PROJECTS_OPEN];
        MultiTrackPanel* multiTrackPanel[MAX_PROJECTS_OPEN];
        AudioSourcesWindow* audioSourcesWindow;
        MustuxAboutWindow* aboutWindow;

        // Widgets
        GtkWidget* projectMenu;
        GtkWidget* miNewProject;
        GtkWidget* miOpenProject;
        GtkWidget* miSaveProject;
        GtkWidget* miCloseProject;
        GtkWidget* audioMenu;
        GtkWidget* miImportAudio;
        GtkWidget* miExportAudio;
        GtkWidget* renderMenu;
        GtkWidget* miRenderTrack;
        GtkWidget* miRenderProject;
        GtkWidget* settingsMenu;
        GtkWidget* miSettings;
        GtkWidget* mustacheMenu;
        GtkWidget* miNewMusic;
        GtkWidget* miOpenMusic;
        GtkWidget* helpMenu;
        GtkWidget* miHelp;
        GtkWidget* miAbout;
        int aboutTag;

        int new_project(int ntracks);
        int import_audio();

        void show_global_settings();

        // callbacks
        ML_DECLARE_CALLBACK(on_new_project);
        ML_DECLARE_CALLBACK(on_open_project);
        ML_DECLARE_CALLBACK(on_save_project);
        ML_DECLARE_CALLBACK(on_close_project);
        ML_DECLARE_CALLBACK(on_import_audio);
        ML_DECLARE_CALLBACK(on_export_audio);
        ML_DECLARE_CALLBACK(on_render_track);
        ML_DECLARE_CALLBACK(on_render_project);
        ML_DECLARE_CALLBACK(on_new_music);
        ML_DECLARE_CALLBACK(on_open_music);
        ML_DECLARE_CALLBACK(on_settings);
        ML_DECLARE_CALLBACK(on_show_help);
        ML_DECLARE_CALLBACK(on_show_about);
        ML_DECLARE_CALLBACK(on_exit);
        };

#include "MustuxLib/MustuxLib.hh"
#include "MultiTrackArea.hh"
#include "MultiTrackPanel.hh"
#include "AudioSourcesWindow.hh"

#endif

