#include <cmath>
#include "Locator.hh"


Locator::Locator(GtkWidget* parent_widget, AudioEditingArea* target, int w, int h)
	{
	PENTER("");
	associated_AEA = target;
	eventwindow = parent_widget;

	GtkWidget* hseparator;
	
	drawarea = gtk_drawing_area_new ();
	gtk_widget_set_events (drawarea, GDK_EXPOSURE_MASK  | GDK_LEAVE_NOTIFY_MASK
					| GDK_BUTTON_PRESS_MASK | GDK_POINTER_MOTION_MASK
					| GDK_POINTER_MOTION_HINT_MASK);
	gtk_drawing_area_size (GTK_DRAWING_AREA(drawarea), w, h);
	gtk_widget_show (drawarea);
	gtk_box_pack_start (GTK_BOX (parent_widget), drawarea, false, false, 0);
	
	hseparator = gtk_hseparator_new ();
	gtk_widget_show (hseparator);
	gtk_box_pack_start(GTK_BOX (parent_widget), hseparator, false, false, 0);
	
	pixmap = gdk_pixmap_new(drawarea->window, drawarea->allocation.width, drawarea->allocation.height ,-1);
	clear();
	JOG_Active = false;		
	shuttle_active = false;
	cursor_position = 0;
	cursor_width = 0;
	shuttle_amount = 0;
	PEXIT("");
	}

	
Locator::~Locator()
	{
	PENTER("");
	if (pixmap) gdk_pixmap_unref(pixmap);
	PEXIT("");
	}

static void _update(GtkWidget *widget, GdkEventExpose *event, Locator *o)
	{
	o->update(0, 0, -1, -1);
	}

static void _locate(GtkWidget *widget, GdkEventButton *event, Locator *o)
	{
	o->locate( (int) event->x);
	}


static int _reconfigure(GtkWidget *widget, GdkEventConfigure *event, Locator *o)
	{
	o->reconfigure();
	return 1;
	}


void Locator::start(Audio* as)
	{
	PENTER("");
	audio_source = as;
	build();
	gtk_signal_connect (GTK_OBJECT (drawarea), "expose_event", GTK_SIGNAL_FUNC (_update), this);
	gtk_signal_connect (GTK_OBJECT (drawarea), "button_press_event", GTK_SIGNAL_FUNC (_locate), this);
	gtk_signal_connect (GTK_OBJECT (drawarea), "configure_event", GTK_SIGNAL_FUNC (_reconfigure), this);
        PEXIT("");
	}

void Locator::build()
	{
	PENTER("");
	static GdkGC* c1 = Colors::get_color(LOCATOR_PEAK_COLOR);
	int width = drawarea->allocation.width;
	int height = drawarea->allocation.height;
	int nch = audio_source->get_number_of_channels();
	int yheight = height / nch;
	int ycenter = yheight / 2;
	float f = (float) (height/nch) / 65536;

	int bs = audio_source->get_blocksize();
	long tb = audio_source->get_total_blocks();
	peak_step =  bs * (int) ( (tb / width) / bs );
        long pos = audio_source->get_headersize();
	FILE* file = audio_source->get_file();
	long originalpos = ftell(file);
	fseek(file, pos, SEEK_SET);

	short sample;
	int y = 0;
	int yc = 0;
	int x = 0;
	while (!feof(file)  &&   ( x < width) )
		{
		for (int k=0; k<nch; k++)
			{
			yc = ycenter+yheight*k;
			fread(&sample, sizeof(short), 1, file);
			y = (int) (f * (float) sample); // make it logarythmic
			gdk_draw_line (pixmap, c1, x, yc , x,  yc - y);
			}
              pos+=peak_step * bs;
		fseek(file, pos, SEEK_SET);
		x++;
		}
	fseek(file, originalpos, SEEK_SET);
	PEXIT("");
	}


void Locator::rebuild()
	{
	PENTER("");
	recreate();
	PEXIT("");
	}

void Locator::update(int x, int y, int w, int h)
	{
	PFLOODENTER("");
	gdk_draw_pixmap(drawarea->window , drawarea->style->white_gc, pixmap, x, y, x, y, w , h);
	PFLOODEXIT("");
	}


void Locator::recreate()
	{
	PENTER("");
	clear();
	build();
	update(0, 0, -1, -1);
        resize_cursor();
	PEXIT("");
	}


void Locator::reconfigure()
	{
	PENTER("");
	if (pixmap) gdk_pixmap_unref(pixmap);
	pixmap = gdk_pixmap_new(drawarea->window, drawarea->allocation.width,
 				   drawarea->allocation.height, -1);
	if (associated_AEA->get_file_status() != NO_AUDIO)
		recreate();
	else
		{
		clear();
		update(0,0,-1,-1);
		}
	PEXIT("");
	}



void inline Locator::clear()
	{
	PENTER("");
	gdk_draw_rectangle (pixmap,  Colors::get_color(LOCATOR_BACKGROUND_COLOR),  true, 0 , 0,
			    drawarea->allocation.width, drawarea->allocation.height);
	PEXIT("");
	}



void Locator::resize_cursor()
	{
	PENTER("");
	toggle_cursor();
	if (peak_step == 0)
		cursor_width = drawarea->allocation.width;
	else
		cursor_width = (int) ( pow(2,associated_AEA->get_hzoom() ) * drawarea->allocation.width / peak_step) + 1;
	toggle_cursor();
	PEXIT(" : cursor width = " << cursor_width);
	}

			
	

void Locator::locate(int new_screen_pos)
	{
	PFLOODENTER("");
	associated_AEA->set_first_blockpos(new_screen_pos * peak_step);
	associated_AEA->recreate();
	toggle_cursor();
	cursor_position = new_screen_pos;
	toggle_cursor();
	update(cursor_position, 0, cursor_width+1, drawarea->allocation.height);
	PFLOODEXIT("");
	}


void Locator::toggle_cursor()
 	{
        gdk_gc_set_function (drawarea->style->fg_gc[GTK_STATE_NORMAL], GDK_INVERT);
	gdk_draw_rectangle (pixmap, drawarea->style->fg_gc[GTK_STATE_NORMAL], false,
				 cursor_position, 0,	 cursor_width, drawarea->allocation.height-2);
	update(cursor_position, 0, cursor_width+1, drawarea->allocation.height);
	gdk_gc_set_function (drawarea->style->fg_gc[GTK_STATE_NORMAL], GDK_COPY);
 	}

 		
void Locator::update_cursor()
 	{
	PFLOODENTER("");
	toggle_cursor(); 	
	if (peak_step ==0)
		cursor_position = 0;
	else
		cursor_position = (int) (associated_AEA->get_first_blockpos() / peak_step);
	toggle_cursor();
 	PFLOODEXIT("");
 	}


// JOG LOCATOR

static int JOG_handler(GtkWidget *widget, GdkEventMotion *event, Locator *o )
	{
	int x, y;
	GdkModifierType state;
	gdk_window_get_pointer (event->window, &x, &y, &state);
	o->locate(x);
	return 1;
	};


void Locator::JOG_locator()
	{
	if (associated_AEA->get_dsp_status()==STOPPED)
		if (!JOG_Active)
			{
            JOG_Active = true;
			JOG_tag = gtk_signal_connect (GTK_OBJECT (eventwindow),
				   "motion_notify_event", (GtkSignalFunc) JOG_handler, this);
			}
		else
			{
			JOG_Active = false;
			gtk_signal_disconnect (GTK_OBJECT (eventwindow), JOG_tag);
			}
	}

bool Locator::jog_is_active()
    {
    return JOG_Active;
    }

// SHUTTLE LOCATOR

static int shuttle_handler(GtkWidget *widget, GdkEventMotion *event, Locator *o )
	{
	int x, y;
	GdkModifierType state;
	gdk_window_get_pointer (event->window, &x, &y, &state);
	o->shuttle_scroll(x);
	return 1;
	};



static int shuttle_timehandler(gpointer obj)
	{
	Locator *o = (Locator*) obj;
	o->shuttle_update();
	return 1;
	}



void Locator::shuttle(int mode)
	{
	PENTER("");
	if (associated_AEA->get_dsp_status() == STOPPED)
		if (!shuttle_active)
			{
            shuttle_active = true;
			shuttle_tag = gtk_signal_connect (GTK_OBJECT (eventwindow),
				   "motion_notify_event", (GtkSignalFunc) shuttle_handler, this);
			shuttle_timetag = gtk_timeout_add( 10 , shuttle_timehandler, this );
			shuttle_mode = mode;
			shuttle_factor = (float) (shuttle_mode * pow(2,associated_AEA->get_hzoom())) / 50;
			}
		else
			{
			shuttle_active = false;
			gtk_signal_disconnect (GTK_OBJECT (eventwindow), shuttle_tag);
			gtk_timeout_remove(shuttle_timetag);
            shuttle_mode = 0;
            }
	PEXIT("");
	}


void Locator::shuttle_scroll(int x)
	{
	PFLOODENTER("");
	int offset = x - drawarea->allocation.width/2;
	if (abs(offset)<10)
		must_shuttle=false;
	else
		must_shuttle=true;
	shuttle_amount = int( offset * shuttle_factor);
	PFLOODEXIT("");
	}



void Locator::shuttle_update()
	{
	PFLOODENTER("");
	if (must_shuttle)
		{
		associated_AEA->set_first_blockpos(associated_AEA->get_first_blockpos() + shuttle_amount);
		associated_AEA->recreate();
		update_cursor();
		}
	PFLOODEXIT("");
	}


bool Locator::shuttle_is_active()
    {
    return shuttle_active;
    }

// eof



