#include "MustuxLib/MustuxLib.hh"
#include "Mixer.hh"
#include "MultiTrackArea.hh"
#include "Track.hh"
#include "PrafFile.hh"



SubClip::SubClip(Audio* anAudioSource, long long aRelativeBlock,  long long aFirstBlock, int  aWidth)
        {
        audioSource = anAudioSource;
        relativeSliceBlock = aRelativeBlock;
        firstFileBlock = aFirstBlock;
        width = aWidth;
        }

SubClip::~SubClip()
        {
        }


void SubClip::rewind()
        {
        audioSource->locate(firstFileBlock);
        }


Slice::Slice( int anID )
        {
        ID = anID;
        firstSubClip=0;
        mixBuffer=0;
        }

Slice::~Slice()
        {
        SubClip* sc = firstSubClip;
        SubClip* sc2= firstSubClip;
        while (sc)
                {
                sc2=sc->next;
                delete sc;
                sc=sc2;
                }
        }


void Slice::set_dsp_parameters(int aRate, int aChannels, int aBitDepth)
        {
        rate = aRate;
        channels = aChannels;
        bitDepth = aBitDepth;
        blockSize = channels * (bitDepth/8);
        }


int Slice::add_sub_clip(SubClip* anSubClip)
        {
        if (!firstSubClip)
                {
                firstSubClip=anSubClip;
                firstSubClip->prev=0;
                firstSubClip->next=0;
                }
        else
                {
                SubClip* p = firstSubClip;
                while (p->next)
                        {
                        p=p->next;
                        }
                p->next=anSubClip;
                anSubClip->next=0;
                anSubClip->prev=p;
                }
        return 1;
        }




char* Slice::fast_mix()
        {
        PFLOODENTER;
        int widthInBytes = (rightBorder-leftBorder) * blockSize;
        PMESG("Mixing Slice #" << ID << " Buffer Size (in bytes) = " << widthInBytes);
        mixBuffer = new char[widthInBytes];

        char *p=mixBuffer ;
        for (int i=0; i<widthInBytes; i++) *(p++)=0;

        SubClip* sc=firstSubClip;
        while (sc)
                {
                sc->audioSource->locate(sc->firstFileBlock);
                FILE* file= sc->audioSource->get_file();
                char* p = mixBuffer + sc->relativeSliceBlock*blockSize;
                int bytesToRead=sc->width*blockSize;
                char* lastp = p + (bytesToRead);
                char* sourcebuffer=new char[bytesToRead];
                fread(sourcebuffer, 1, bytesToRead, file);
                char* sp = sourcebuffer;
                while (p<lastp)
                        {
                        *(p++)+=*(sp++);// SIMPLE ADDITION MIX TECHNIQUE : NO NORMALIZATION
                        }
                sc=sc->next;
                }
        PFLOODEXIT;
        return (char*) mixBuffer;
        }




char* Slice::render()
        {
        PFLOODENTER;
        int widthInBytes = (rightBorder-leftBorder) * blockSize;
        //PMESG("Rendering Slice #" << ID << " Buffer Size (in bytes) = " << widthInBytes);
        mixBuffer = new char[widthInBytes];

        char *p=mixBuffer ;
        for (int i=0; i<widthInBytes; i++) *(p++)=0;

        SubClip* sc=firstSubClip;
        while (sc)
                {
                sc->audioSource->locate(sc->firstFileBlock);
                FILE* file= sc->audioSource->get_file();
                char* p = mixBuffer + sc->relativeSliceBlock*blockSize;
                int bytesToRead=sc->width*blockSize;
                char* lastp = p + (bytesToRead);
                char* sourcebuffer=new char[bytesToRead];
                fread(sourcebuffer, 1, bytesToRead, file);
                char* sp = sourcebuffer;
                while (p<lastp)
                        {
                        *(p++)+=*(sp++);// SIMPLE ADDITION MIX TECHNIQUE : NO NORMALIZATION
                        }
                sc=sc->next;
                }
        PFLOODEXIT;
        return (char*) mixBuffer;
        }



void Slice::release_mix()
        {
        if (mixBuffer)
                delete mixBuffer;
        }





void Slice::show()
        {
        PMESG("*** Slice " << ID << " SubClips are :");
        SubClip* sc = firstSubClip;
        while (sc)
                {
                PMESG("   SubClip  " << sc <<" : RSP =" << sc->relativeSliceBlock <<" , Width = " << sc->width << " , FSB =" << sc->firstFileBlock);
                sc=sc->next;
                }
        }



long long Slice::get_width_in_bytes()
        {
        return (width) * blockSize;
        }




// ---------------------------------  MIXER CLASS

Mixer::Mixer(MultiTrackArea* aMta)
        {
        mta = aMta;
        rate = 44100;
        channels = 2;
        bitDepth = 16;
        blockSize = channels * (bitDepth/8);
        firstSlice = 0 ;
        dspFragSize = DEFAULT_DSP_FRAGMENT_SIZE;
        }



Mixer::~Mixer()
        {
        Slice* s = firstSlice;
        Slice* s2= firstSlice;
        while (s)
                {
                s2=s->next;
                delete s;
                s=s2;
                }
        }


void Mixer::set_rate(int aRate)
        {
        rate = aRate;
        }


void Mixer::set_channels(int aChannels)
        {
        channels = aChannels;
        }


void Mixer::set_bit_depth(int aBitDepth)
        {
        bitDepth = aBitDepth ;
        }


static int play_dsp_frag(gpointer obj)
        {
        Mixer* o = (Mixer*) obj;
        // printf("S# %d  : mB = %p  mBsize=%d  bP=%p  ebP=%p \n",o->currentSlice->ID,o->mixedBuffer,o->mixedBufferSize,o->bufPointer,o->endbufPointer);
        if (o->bufPointer < o->endbufPointer)
                {
                o->dsp->play_fragment(o->bufPointer , o->dspFragSize);
                o->bufPointer += o->dspFragSize;
                o->relativeBlockInsideSliceBeingPlayed += (o->dspFragSize / o->blockSize);
                if (o->bufPointer >= o->endbufPointer)
                        {
                        o->next_slice();
                        }
                }
        return 1;
        }
        

int Mixer::start_play()
        {
        PENTER;

        PMESG("Starting DSP Engine ...");
        dsp = new Dsp();
        if ( dsp->open_for_play() < 0)
                {
                PERROR("Opening DSP for PLAY failed.");
                PEXIT;
                return -1;
                }
        if ( dsp->set(channels, rate , bitDepth) < 0 )
                {
                PERROR("Setting DSP Parameters failed.");
                PEXIT;
                return -1;
                }
        if ( dsp->set_format(AFMT_S16_LE) < 0 )
                {
                PERROR("Setting DSP Sample Format failed.");
                PEXIT;
                return -1;
                }
        dspFragSize = dsp->get_best_fragment_size();
        PMESG("Detected Best dspFragSize = " << dspFragSize);

        build_slice_list();

        goto_first_slice();
        playTag = gtk_timeout_add( 5, play_dsp_frag, this ); // gtk_timeout_add could be a MustuxKernel Macro
        PEXIT;
        return 1;
        }



int Mixer::stop_play()
        {
        PENTER;
        PMESG("Stopping DSP Engine ...");
        if (dsp->close()<0)
                return -1;
        gtk_timeout_remove(playTag);
        mta->isPlaying=false;
        PEXIT;
        return 1;
        }


int Mixer::build_slice_list()
        {
        PENTER;
        // find the last blockpos in MT
        int tt = mta->get_total_tracks();
        long long lastBlockPos=0;
        for (int i=0; i<tt; i++)
                {
                AudioClip* clip = mta->track[i]->clipList;
                while (clip != 0)
                        {
                        long long borderR = clip->trackFirstBlock + (clip->sourceLastBlock - clip->sourceFirstBlock);
                        if (borderR>=lastBlockPos)
                                lastBlockPos = borderR;
                        clip=clip->next;
                        }
                }

        firstSlice = 0;
        int sliceWidth = dspFragSize*SLICE_WIDTH_FACTOR;
        PMESG("Starting building Slice List : Width = " << sliceWidth);
        int ID=0;
        long long border = 0;
        bool first = true;
        Slice* s;
        while (lastBlockPos > border)
                {
                if (first)
                        {
                        s = new Slice(ID);
                        firstSlice=s;
                        firstSlice->prev=0;
                        first=false;
                        }
                else
                        {
                        s->next = new Slice(ID);
                        s->next->prev=s;
                        s=s->next;
                        }
                s->set_dsp_parameters( rate, channels, bitDepth );
                s->leftBorder=border;
                s->rightBorder=border+sliceWidth;
                s->width=sliceWidth;
                //PMESG("Created Slice #" << ID << " LB = " << s->leftBorder << " ,  RB = " << s->rightBorder);
                ID++;
                border+=sliceWidth;
                }
        s->next=0;


        PMESG("Finding sub-clips inside Slices...");
        Slice* slice = firstSlice;
        int nsc = 0;
        while (slice)
                {
                long long sliceLBorder = slice->leftBorder;
                long long sliceRBorder = slice->rightBorder;
                int sliceWidth = sliceRBorder - sliceLBorder;
                int rsp;
                for (int i=0; i<tt; i++)
                        {
                        AudioClip* clip = mta->track[i]->get_clip_under(sliceLBorder);
                        if (clip)
                                {
                                SubClip* sc;
                                long long sfb = clip->sourceFirstBlock + (sliceLBorder-clip->trackFirstBlock);
                                rsp=0;
                                if (clip->trackLastBlock>sliceRBorder)
                                        {
                                        int subclipWidth=sliceWidth;
                                        //PMESG("Found FULL Subclip in Slice #" << slice->ID << " under track " << i << " RSP=0 , SFB=" << sfb << " , Width=" << subclipWidth);
                                        sc = new SubClip(clip->audioSource, rsp, sfb, subclipWidth);
                                        }
                                else
                                        {
                                        int subclipWidth = clip->trackLastBlock - sliceLBorder;
                                        //PMESG("Found Subclip in Slice #" << slice->ID << " under track " << i << " RSP=0 , SFB=" << sfb << " , Width=" << subclipWidth);
                                        sc = new SubClip(clip->audioSource, rsp, sfb, subclipWidth);
                                        }
                                slice->add_sub_clip(sc);
                                nsc++;
                                }
                        clip = mta->track[i]->get_next_clip_after(sliceLBorder);
                        while (clip && (clip->trackFirstBlock < sliceRBorder))
                                {
                                int rsp = clip->trackFirstBlock - sliceLBorder;
                                int sfb = clip->sourceFirstBlock;
                                if (clip->trackLastBlock < sliceRBorder)
                                        {
                                        int subclipWidth = clip->lenght;
                                        //PMESG("Found Subclip in Slice #" << slice->ID << " under track " << i << " RSP=" << rsp << " , SFB=" << sfb << " , Width=" << subclipWidth);
                                        SubClip* sc = new SubClip(clip->audioSource, rsp, sfb, subclipWidth);
                                        slice->add_sub_clip(sc);
                                        nsc++;
                                        clip = mta->track[i]->get_next_clip_after(clip->trackLastBlock);
                                        }
                                else
                                        {
                                        int subclipWidth = sliceRBorder-clip->trackFirstBlock;
                                        //PMESG("Found Subclip in Slice #" << slice->ID << " under track " << i << " RSP=" << rsp << " , SFB=" << sfb << " , Width=" << subclipWidth);
                                        SubClip* sc = new SubClip(clip->audioSource, rsp, sfb, subclipWidth);
                                        slice->add_sub_clip(sc);
                                        nsc++;
                                        break;
                                        }
                                }
                        }
                slice=slice->next;
                }
        PMESG("Found " << nsc << " subclips inside slices");
        // sliceList->show();
        PEXIT;
        return 1;
        }


void Mixer::show_slice_list()
        {
        PMESG("Slice List :");
        Slice* s = firstSlice;
        while (s)
                {
                s->show();
                s=s->next;
                }
        }






void Mixer::goto_first_slice()
        {
        PENTER;
        currentSlice = firstSlice;
        mixedBuffer = currentSlice->fast_mix();  // I am mixing the Slice right in the time I need it to be played.
                                                // this is wrong! I have to put a separeted thread to mix the "next of the next" everytime
                                               // I reach the "next". By other hand, I cannot make a syncronous mix.
                                              // the slice->mix() method must be such that make the /n algorithm in a separeted thread,
        mixedBufferSize = currentSlice->get_width_in_bytes();
        bufPointer = (char*) mixedBuffer;
        endbufPointer = bufPointer + mixedBufferSize;
        relativeBlockInsideSliceBeingPlayed = 0;
        PEXIT;
        }



void Mixer::next_slice()
        {
        PENTER;
        currentSlice->release_mix();
        currentSlice = currentSlice->next;
        PMESG("Moving to next Slice ..." << currentSlice);
        if (!currentSlice)
                {
                stop_play();
                }
        else
                {
                mixedBuffer = currentSlice->fast_mix();
                mixedBufferSize = currentSlice->get_width_in_bytes();
                bufPointer = (char*) mixedBuffer;
                endbufPointer = bufPointer + mixedBufferSize;
                relativeBlockInsideSliceBeingPlayed = 0;
                }
        PEXIT;
        }


long long Mixer::get_current_playing_block()
        {
        return currentSlice->leftBorder + relativeBlockInsideSliceBeingPlayed;
        }


int Mixer::render_all()
        {
        PENTER;
        MustuxProgressBar* pb = new MustuxProgressBar();
        pb->build();
        pb->init("Rendering all Tracks to MTAoutput.praf");
        pb->flush_events();
        pb->configure(0.0 , 100.0 , 0.0);

        build_slice_list();

        Slice* sl = firstSlice;

        int ns = 0;
        while (sl) { ns++; sl=sl->next; }
        // PMESG("Found " << ns << " slices to render");

        PrafFile* pf = new PrafFile();
        pf->create("MTAoutput.praf");
        pf->write_header(channels, rate, bitDepth, 0,
                         "RenderAllTest", "Test", "Test",
                         "Output audio from RenderAll action");


        sl = firstSlice;
        int i=0;
        while (sl)
                {
                float f = ((float) i / ns);
                mixedBuffer = sl->render();
                mixedBufferSize = sl->get_width_in_bytes();
                pf->append_samples_from_buffer( mixedBuffer, mixedBufferSize);
                sl->release_mix();
                sl=sl->next;
                pb->set_value(f*100.0);
                pb->update();
                i++;
                }
        pf->fix_audio_size();
        pf->close_file();
        delete pf;


        delete pb;
        PEXIT;
        return 1;
        }
//eof

