#ifndef MIXER_HH
#define MIXER_HH

#include <cstdio>
#include <string>

class Audio;
class MultiTrackArea;
class Dsp;

#define SLICE_WIDTH_FACTOR 16


class SubClip
        {
        public :
                SubClip* next;
                SubClip* prev;
                Audio* audioSource;

                long long firstFileBlock;
                long long relativeSliceBlock;
                int  width;

                SubClip(Audio* anAudioSource,
                        long long aRelativeBlock,
                        long long aFirstBlock,
                        int aWidth);
                ~SubClip();

                void rewind();
        };



class Slice
        {
        public :
                int ID;
                Slice *prev;
                Slice *next;
                long long leftBorder;
                long long rightBorder;
                SubClip* firstSubClip;
                char* mixBuffer;
                int rate;
                int channels;
                int bitDepth;
                int blockSize;
                long long width;
                long long get_width_in_bytes();

                Slice(int anID);
                ~Slice();
                void set_dsp_parameters(int aRate, int aChannels, int aBitDepth);
                int add_sub_clip(SubClip* sc);
                char* fast_mix();
                char* render();
                int add(SubClip* anSubClip);
                void release_mix();
                void show();

        };



class Mixer
        {
        public:

                static const int DEFAULT_DSP_FRAGMENT_SIZE = 4096;
                Mixer(MultiTrackArea* aMta);
                ~Mixer();
                int dspFragSize;
                int playTag;
                char* bufPointer;
                char* endbufPointer;
                Dsp* dsp;
                Slice* currentSlice;
                char* mixedBuffer;
                int mixedBufferSize;
                int rate;
                int channels;
                int bitDepth;
                int blockSize;
                int relativeBlockInsideSliceBeingPlayed;

                void set_rate(int aRate);
                void set_channels(int aChannels);
                void set_bit_depth(int aBitDepth);
                int render_all();

                long long get_current_playing_block();

                MultiTrackArea* mta;
                Slice* firstSlice;
                int build_slice_list();
                void show_slice_list();
                int start_play();
                int stop_play();
                void goto_first_slice();
                void next_slice();
        };


#include "Audio.hh"
#include "MultiTrackArea.hh"
#include "Dsp.hh"

#endif


