#include <iostream>
#include <strstream>
#include <string>
#include <cmath>
#include "MustuxLib/MustuxLib.hh"
#include "MultiTrackArea.hh"
#include "Mixer.hh"
#include "Track.hh"
#include "AudioClip.hh"


MultiTrackArea::MultiTrackArea(MustuxComponent* parent) : MustuxPane (parent)
        {
        PENTERCONS;
        PEXITCONS;
        }

MultiTrackArea::MultiTrackArea(GtkWidget* aParentBox) : MustuxPane (aParentBox)
        {
        PENTERCONS;
        PEXITCONS;
        }


MultiTrackArea::~MultiTrackArea()
        {
        PENTERDES;
        PEXITDES;
        }


int MultiTrackArea::build(int aTotalTracks)
        {
        PENTER;
        build(aTotalTracks, DEFAULT_MTA_WIDTH, DEFAULT_MTA_HEIGHT);
        PEXIT;
        return 1;
        }


int MultiTrackArea::build(int aTotalTracks, int aWidth, int aHeight)
        {
        PENTER;
        totalTracks=aTotalTracks;
        MustuxPane::build( aWidth, aHeight,MustuxComponent::RESIZEABLE);
        if (totalTracks > MAX_TRACKS)
                {
                PERROR("Max Tracks = " << MAX_TRACKS);
                PEXIT;
                return -1;
                }

        int trackHeight = aHeight / totalTracks;
        for (int i=0; i<totalTracks; i++)
                {
                track[i]= new Track(this, 0, i*trackHeight, aWidth, trackHeight);
                track[i]->ID = i;
                }

        PEXIT;
        return 1;
        }


int MultiTrackArea::init()
        {
        PENTER;
        MustuxPane::init();
        for (int zoom=0; zoom<MAX_ZOOM_LEVELS; zoom++)
                zoomStep[zoom] = (int) pow(2,zoom);

        activeTrack=0;
        isFloatCursorHidden = false;
        isWorkCursorHidden = false;
        isScrolling = false; //?
        isPlaying = false;
        isRecording = false;
        isJogSliding = false;
        hzoom = 9;// FOR LATER :GlobalSettings::get_default_hzoom();
        vzoom = 1;// FOR LATER :GlobalSettings::get_default_vzoom();
        floatCursorXpos=0;
        workCursorXpos=0;

        firstBlock = 0;
        lastBlock = -1;
        workingBlock = 0;
        playingBlock = 0;
        activeTrack=0;

        for (int i=0; i<totalTracks; i++)
                {
                track[i]->recreate();
                }

        track[activeTrack]->activate();
        enable_float_cursor();
        show_float_cursor();
        show_work_cursor();


        PEXIT;
        return 1;
        }


int MultiTrackArea::finalize()
        {
        PENTER;
        MustuxPane::finalize();
        PEXIT;
        return 1;
        }


int MultiTrackArea::unbuild()
        {
        PENTER;
        for (int i=0; i<MAX_TRACKS; i++)
                delete track[i];
        MustuxPane::unbuild();
        PEXIT;
        return 1;
        }


void MultiTrackArea::recreate()
        {
        PENTER;
        toggle_float_cursor();
        toggle_work_cursor();
        int trackHeight = get_height() / totalTracks;
        for (int i=0; i<totalTracks; i++)
                {
                track[i]->baseX = 0;
                track[i]->baseY = i * trackHeight ;
                track[i]->width = get_width();
                track[i]->height = trackHeight;
                track[i]->recreate();
                }
        toggle_work_cursor();
        toggle_float_cursor();
        PEXIT;
        }




int MultiTrackArea::fill_track(string filename)
    {
    PENTER;
    hide_float_cursor();
    hide_work_cursor();
    Audio* au = new Audio();
    int r = au->build(filename);
    if (r)
        {
        if (track[activeTrack]->fill(workingBlock, au)>0)
                {
                PMESG("track " << activeTrack << " sucessfully filled.");
                }
        }
    else
        {
        PERROR("File not Found : " << filename);
        }
    show_work_cursor();
    show_float_cursor();
    PEXIT;
    return r;
    }


void MultiTrackArea::render_all()
        {
        PENTER;
        mixer = new Mixer(this);
        // try to guess rate, channels, bitDepth
        // This is a dumb algorithm, since it takes the dsp parameters from the first clip it finds
        for (int i=0; i<totalTracks; i++)
                {
                AudioClip* c= track[i]->clipList;
                if (c)
                        {
                        mixer->set_rate(c->get_audio_source()->get_rate());
                        mixer->set_channels(c->get_audio_source()->get_channels());
                        mixer->set_bit_depth(c->get_audio_source()->get_bit_depth());
                        break;
                        }
                }
        int r = mixer->render_all();
        if (r<0)
                {
                PERROR("Rendering failed.");
                }
        delete mixer;
        PEXIT;
        }

int MultiTrackArea::split()
    {
    PENTER;
    hide_float_cursor();
    disable_float_cursor();
    touch();
    track[activeTrack]->split_clip(workingBlock);
    track[activeTrack]->recreate();
    enable_float_cursor();
    show_float_cursor();
    PEXIT;
    return 1;
    }


void MultiTrackArea::toggle_play_stop()
        {
        if (isPlaying)
                stop();
        else
                play();
        }


static int _update_play_cursor(gpointer obj)
        {
        MultiTrackArea *o = (MultiTrackArea*) obj;
        o->update_play_cursor();
        return 1;
        }


int MultiTrackArea::play()
    {
    PENTER;
    mixer = new Mixer(this);

    // try to guess rate, channels, bitDepth
    // This is a dumb algorithm, since it takes the dsp parameters from the first clip it finds
    for (int i=0; i<totalTracks; i++)
        {
        AudioClip* c= track[i]->clipList;
        if (c)
                {
                mixer->set_rate(c->get_audio_source()->get_rate());
                mixer->set_channels(c->get_audio_source()->get_channels());
                mixer->set_bit_depth(c->get_audio_source()->get_bit_depth());
                break;
                }
        }
    if (mixer->start_play()<0)
        {
        PEXIT;
        return -1;
        }
    isPlaying=true;
    followTag = gtk_timeout_add( 50, _update_play_cursor, this ); // 20 fps
    show_play_cursor();
    PEXIT;
    return 1;
    }




void MultiTrackArea::update_play_cursor()
        {
        PFLOODENTER;
        if (!isPlaying)
                        {
                        PMESG("Mixer seems to be stopped");
                        stop();
                        /*stop_blinking_icon();
                        scrolling = false;
                        LCD2->clear();
                        LCD2->draw_background_grid();
                        LCD2->draw_icon(PAUSE_ICON);
                        LCD2->update();*/
                        }
        long long lo = mixer->get_current_playing_block();
        int x = block_to_xpos(lo);
        // PMESG("mixer->get_current_playing_block() = " << lo << " ==> xpos = " << x);
        /*
        if ((playing_screenpos > drawarea->allocation.width) && (!scrolling))
                {
                scrolling = true;
                correction_bias = 0;
                }
        if (scrolling)
                {
                if ( playing_screenpos > drawarea->allocation.width / 2)
                        correction_bias++;
                int amount = playing_screenpos - drawarea->allocation.width + correction_bias;
                first_blockpos += amount * zoom_step[hzoom];
                // the next line is stupid. why recreate everything ?
                // partial_recreate_from(drawarea->allocation.width - amount);  isnt it a better solution ? :)
                recreate();
                }
        }
        */
        if ((x >0) && (x <= get_width()))
                {
                toggle_play_cursor();
                play_xpos = x;
                toggle_play_cursor();
                }
        PFLOODEXIT;
        }




int MultiTrackArea::stop()
        {
        PENTER;
        mixer->stop_play();
        delete mixer;
        isPlaying=false;
        gtk_timeout_remove(followTag);
        hide_play_cursor();
        PEXIT;
        return 1;
        }




void MultiTrackArea::hzoom_in()
    {
    PENTER;
    if (hzoom > 0)
        {
        hzoom--;
        PMESG("HZOOM Now is " << hzoom);
        recreate();
        }
    center();
    PEXIT;
    }



void MultiTrackArea::hzoom_out()
    {
    PENTER;
    if ( hzoom < MAX_ZOOM_LEVELS - 1 )
            {
            hzoom++;
            PMESG("HZOOM Now is " << hzoom);
            recreate();
            }
    center();
    PEXIT;
    }



void MultiTrackArea::vzoom_out()
    {
    if (!isPlaying)
            {
            if (vzoom > 1) vzoom--;
            recreate();
            }
    }



void MultiTrackArea::vzoom_in()
    {
    if (!isPlaying)
            {
            if (vzoom < 5) vzoom++;
            recreate();
            }
    }



void MultiTrackArea::goto_begin()
    {
    if (isPlaying)
        {
        firstBlock = 0;
        workingBlock = 0;
        playingBlock = 0;
        //for (int i=0; i<MAX_TRACKS; i++)
        //    track[i]->rewind();
        recreate();
        }
    }



void MultiTrackArea::goto_end()
    {
    PENTER;
    if ( !isPlaying && !isRecording )
            {
            //firtBlock = ...
            //for (int i=0; i<MAX_TRACKS; i++)
                //workingBlock = track[i]->get_max_block();
            //center();
            }
    PEXIT;
    }


void MultiTrackArea::center()
    {
    PENTER;
    if (!isPlaying && !isRecording)
            {
            long long minimumBlockToCenter = get_width() * zoomStep[hzoom] / 2;
            if ( workingBlock >= minimumBlockToCenter )
                    {
                    long long half = ( lastBlock - firstBlock ) / 2;
                    firstBlock = workingBlock - half;
                    workingBlock = firstBlock + half;
                    playingBlock = workingBlock;
                    recreate();
                    }
            }
    PEXIT;
    }


void MultiTrackArea::touch()
        {
        PENTER;
        int x = get_mouse_x();
        for (int i=0; i<totalTracks; i++)
                {
                if (track[i]->is_pointed())
                        {
                        update_work_cursor(x);
                        workingBlock = xpos_to_block(x);
                        // If user prefers to activate track on ALL touches ----
                        PMESG("track " << i << " activated.");
                        track[activeTrack]->deactivate();
                        activeTrack=i;
                        track[i]->activate();
                        //------------------------------------------------------
                        PEXIT;
                        return ;
                        }
                }
        PEXIT;
        }


Track* MultiTrackArea::get_track_under_Y (int y)
        {
        for (int t=0; t<totalTracks; t++)
                {
                if ( (y >= track[t]->baseY ) && ( y <= track[t]->baseY + track[t]->height) )
                        return track[t];
                }
        return 0;
        }


int MultiTrackArea::jog_slide()
        {
        PENTER;
        if (!isJogSliding)
                {
                disable_float_cursor();
                hide_float_cursor();
                hide_work_cursor();
                enable_jog_slide_cursor();
                bool validPoint = false;
                int pointedTrack = -1;
                AudioClip* cli = 0 ;
                for (int i=0; i < totalTracks; i++)
                        {
                        if (track[i]->is_pointed())
                                {
                                validPoint = true;
                                pointedTrack = i;
                                }
                        }
                if (validPoint)
                        {
                        int x = get_mouse_x();
                        int pointedBlock = xpos_to_block(x);
                        cli = track[pointedTrack]->get_clip_under(pointedBlock);
                        }
                if (cli)
                        {
                        isJogSliding = true;
                        jogSlidingClip = cli;
                        int cx1 = block_to_xpos( cli->trackFirstBlock );
                        int cx2 = block_to_xpos( cli->trackLastBlock );
                        int cy1 = track[pointedTrack]->baseY;
                        jogSlideCursorX = get_mouse_x();
                        jogSlideCursorY = get_mouse_y();
                        jogSlideCursorW = cx2 - cx1;
                        jogSlideCursorH = track[pointedTrack]->height-5;
                        int x = get_mouse_x();
                        int y = get_mouse_y();
                        jogSlideCursorXOffSet = x - cx1;
                        jogSlideCursorYOffSet = y - cy1;
                        }

                }
        else
                {
                isJogSliding = false;
                int x = get_mouse_x();
                int y = get_mouse_y();
                long long newInsertBlock = xpos_to_block( x - jogSlideCursorXOffSet);
                Track* originTrack = jogSlidingClip->parentTrack;
                Track* targetTrack = get_track_under_Y(y);
                if (!targetTrack)
                        {
                        PMESG("Deleting clip #" << jogSlidingClip);
                        originTrack->delete_clip(jogSlidingClip);
                        }
                else
                        {
                        PMESG("Moving clip #" << jogSlidingClip
                              << " from [Trk:" << originTrack->ID << ",TFB:" << jogSlidingClip->trackFirstBlock << "]"
                              << " to   [Trk:" << targetTrack->ID << ",TFB:" << newInsertBlock << "]");
                        if (originTrack==targetTrack)
                                {
                                jogSlidingClip->set_track_first_block(newInsertBlock);
                                }
                        else
                                {
                                targetTrack->add_clip(jogSlidingClip->audioSource,
                                                      newInsertBlock,
                                                      jogSlidingClip->sourceFirstBlock,
                                                      jogSlidingClip->lenght);
                                originTrack->delete_clip(jogSlidingClip);
                                }
                        targetTrack->recreate();
                        }
                originTrack->recreate();
                disable_jog_slide_cursor();
                show_work_cursor();
                show_float_cursor();
                enable_float_cursor();
                }

        PEXIT;
        return isJogSliding;
        }



void MultiTrackArea::enable_float_cursor()
        {
        PENTER;
        // this solution for mouse motion (float cursor ) is a bit messy. think in something else ...
        motionTag = gtk_signal_connect (GTK_OBJECT (box),
                                       "motion_notify_event",
                                       GTK_SIGNAL_FUNC (on_float_cursor_motion),
                                       this);
        PEXIT;
        }



void MultiTrackArea::disable_float_cursor()
        {
        PENTER;
        gtk_signal_disconnect (GTK_OBJECT (box), motionTag);
        PEXIT;
        }



void MultiTrackArea::enable_jog_slide_cursor()
        {
        PENTER;
        motionTag = gtk_signal_connect (GTK_OBJECT (box),
                                       "motion_notify_event",
                                       GTK_SIGNAL_FUNC (on_jog_slide_motion),
                                       this);

        PEXIT;
        }



void MultiTrackArea::disable_jog_slide_cursor()
        {
        PENTER;
        gtk_signal_disconnect (GTK_OBJECT (box), motionTag);
        PEXIT;
        }


int MultiTrackArea::block_to_xpos(long long block)
    {
    int x=((block - firstBlock)  / zoomStep[hzoom]);
    return x;
    }



long long MultiTrackArea::xpos_to_block(int xpos)
    {
    long long b = firstBlock + xpos * zoomStep[hzoom];
    return b;
    }



void MultiTrackArea::scroll_right()
    {
    PENTER;
    firstBlock +=  xpos_to_block(50);
    recreate();
    PEXIT;
    }


void MultiTrackArea::scroll_left()
    {
    PENTER;
    firstBlock -=  xpos_to_block(50);
    if (firstBlock<0) firstBlock=0;
    recreate();
    PEXIT;
    }


int MultiTrackArea::get_total_tracks()
    {
    return totalTracks;
    }




// get / set stuff


bool MultiTrackArea::is_playing()
    {
    return isPlaying;
    }


bool MultiTrackArea::is_recording()
    {
    return isPlaying;
    }


int MultiTrackArea::get_hzoom()
    {
    return hzoom;
    }


int MultiTrackArea::get_vzoom()
    {
    return vzoom;
    }

int MultiTrackArea::get_zoom_step()
    {
    return zoomStep[hzoom];
    }


long long MultiTrackArea::get_first_block()
    {
    return firstBlock;
    }


long long MultiTrackArea::get_last_block()
    {
    return lastBlock;
    }


void MultiTrackArea::set_first_block(long long pos)
    {
    firstBlock = (pos >=0) ? pos : 0;
    lastBlock = xpos_to_block(1024/* OOOOOPS .. same case.. get_width()*/);
    }

inline void MultiTrackArea::toggle_float_cursor()
    {
    gdk_gc_set_function (drawArea->style->fg_gc[GTK_STATE_NORMAL], GDK_INVERT);
    gdk_draw_line (storePane, drawArea->style->fg_gc[GTK_STATE_NORMAL],
                   floatCursorXpos , 0, floatCursorXpos, get_height());
    gdk_gc_set_function (drawArea->style->fg_gc[GTK_STATE_NORMAL], GDK_COPY);
    update(floatCursorXpos-1,0,2,-1);
    }



inline void MultiTrackArea::toggle_work_cursor()
    {
    gdk_gc_set_function (drawArea->style->fg_gc[GTK_STATE_NORMAL], GDK_INVERT);
    int h = get_height();
    int y=0;
    while (y<h)
        {
        gdk_draw_line (storePane, drawArea->style->fg_gc[GTK_STATE_NORMAL],
                           workCursorXpos , y, workCursorXpos, y+5);
        y+=10;
        }
    gdk_gc_set_function (drawArea->style->fg_gc[GTK_STATE_NORMAL], GDK_COPY);
    update(workCursorXpos-3,0,6,-1);
    }


void MultiTrackArea::toggle_play_cursor()
        {
        gdk_gc_set_function (drawArea->style->fg_gc[GTK_STATE_NORMAL], GDK_INVERT);
        gdk_draw_line (storePane, drawArea->style->fg_gc[GTK_STATE_NORMAL],
                        play_xpos , 0,
                        play_xpos , get_height());
        gdk_gc_set_function (drawArea->style->fg_gc[GTK_STATE_NORMAL], GDK_COPY);
        update(play_xpos - 1, 0, 2,-1);
        }


void MultiTrackArea::update_float_cursor()
        {
        int x = get_mouse_x();
        toggle_float_cursor();
        floatCursorXpos = x;
        toggle_float_cursor();
        }


void MultiTrackArea::toggle_jog_slide_cursor()
        {
        gdk_gc_set_function (drawArea->style->fg_gc[GTK_STATE_NORMAL], GDK_INVERT);
        for (int i=0; i<1; i++)
                gdk_draw_rectangle (storePane, drawArea->style->fg_gc[GTK_STATE_NORMAL], false,
                                    jogSlideCursorX - jogSlideCursorXOffSet + i,
                                    jogSlideCursorY - jogSlideCursorYOffSet + i,
                                    jogSlideCursorW - i*2,
                                    jogSlideCursorH - i*2);
        gdk_gc_set_function (drawArea->style->fg_gc[GTK_STATE_NORMAL], GDK_COPY);
        update(jogSlideCursorX  - jogSlideCursorXOffSet, jogSlideCursorY  - jogSlideCursorYOffSet, jogSlideCursorW+1, jogSlideCursorH+1);
        }


void MultiTrackArea::update_jog_slide_cursor()
        {
        toggle_jog_slide_cursor();
        jogSlideCursorX = get_mouse_x();
        jogSlideCursorY = get_mouse_y();
        toggle_jog_slide_cursor();
        }


void MultiTrackArea::update_work_cursor(int xpos)
    {
    toggle_work_cursor();
    workCursorXpos = xpos;
    toggle_work_cursor();
    }


void MultiTrackArea::show_float_cursor()
    {
    PENTER;
    if (isFloatCursorHidden)
        {
        toggle_float_cursor();
        isFloatCursorHidden = false;
        }
    PEXIT;
    }


void MultiTrackArea::hide_float_cursor()
    {
    PENTER;
    if (!isFloatCursorHidden)
        {
        toggle_float_cursor();
        isFloatCursorHidden = true;
        }
    PEXIT;
    }


void MultiTrackArea::show_play_cursor()
    {
    PENTER;
    toggle_play_cursor();
    PEXIT;
    }

void MultiTrackArea::hide_play_cursor()
    {
    PENTER;
    toggle_play_cursor();
    PEXIT;
    }

void MultiTrackArea::show_work_cursor()
    {
    PENTER;
    if (isFloatCursorHidden)
        {
        toggle_work_cursor();
        isWorkCursorHidden = false;
        }
    PEXIT;
    }


void MultiTrackArea::hide_work_cursor()
    {
    PENTER;
    if (!isFloatCursorHidden)
        {
        toggle_work_cursor();
        isWorkCursorHidden = true;
        }
    PEXIT;
    }

// Callbacks -------------------------------------------------------------------------

int MultiTrackArea::on_float_cursor_motion(GtkWidget *widget, GdkEventMotion *event, MultiTrackArea *o )
    {
    o->update_float_cursor();
    return 1;
    }




int MultiTrackArea::on_jog_slide_motion(GtkWidget *widget, GdkEventMotion *event, MultiTrackArea *o )
    {
    o->update_jog_slide_cursor();
    return 1;
    }


// eof

