#ifndef MULTITRACKAREA_HH
#define MULTITRACKAREA_HH

#include <string>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include "MustuxLib/MustuxLib.hh"

class Mixer;
class Track;
class AudioClip;


class MultiTrackArea : public MustuxPane
        {
        public :
                static const int MAX_TRACKS = 32;
                static const int MAX_ZOOM_LEVELS = 15;
                static const int DEFAULT_MTA_WIDTH = 800;
                static const int DEFAULT_MTA_HEIGHT = 300;

                MultiTrackArea(MustuxComponent* parent);
                MultiTrackArea(GtkWidget* aParentBox);
                ~MultiTrackArea();

                int build(int numberTracks);
                int build(int numberTracks, int width, int height);
                int init();
                int finalize();
                int unbuild();
                void recreate();

                void toggle_play_stop();
                int play();
                int stop();
                int fill_track(string audioFilename);
                int split();
                void hzoom_in();
                void hzoom_out();
                void vzoom_in();
                void vzoom_out();
                void goto_begin();
                void goto_end();
                void center();
                void touch();
                int jog_slide();
                bool is_playing();
                bool is_recording();
                int get_hzoom();
                int get_vzoom();
                int get_zoom_step();
                int get_total_tracks();
                int get_track_height(int trackNumber);
                void scroll_left();
                void scroll_right();
                long long get_first_block();
                long long get_last_block();
                void set_first_block(long long pos);
                int block_to_xpos(long long block);
                long long xpos_to_block(int xpos);
                void show_play_cursor();
                void hide_play_cursor();
                void update_play_cursor();
                void toggle_play_cursor();
                void render_all();

                Track* track[MAX_TRACKS];
                Track* get_track_under_Y (int y);
                friend class Mixer;
                friend class Track;
                friend class AudioClip;

        private :

                Mixer* mixer;
                int totalTracks;
                int activeTrack;
                int hzoom;
                int vzoom;

                long long firstBlock;
                long long lastBlock;
                long long workingBlock;
                long long playingBlock;
                int floatCursorXpos;
                int workCursorXpos;
                int play_xpos;
                int followTag;

                AudioClip* jogSlidingClip;
                int jogSlideCursorX;
                int jogSlideCursorY;
                int jogSlideCursorXOffSet;
                int jogSlideCursorYOffSet;
                int jogSlideCursorW;
                int jogSlideCursorH;

                bool isScrolling;
                bool isPlaying;
                bool isRecording;
                bool isFloatCursorHidden;
                bool isWorkCursorHidden;
                bool isJogSliding;

                int zoomStep[MAX_ZOOM_LEVELS];


                void toggle_work_cursor();
                void update_work_cursor(int x);
                void toggle_float_cursor();
                void update_float_cursor();
                void toggle_jog_slide_cursor();
                void enable_float_cursor();
                void update_jog_slide_cursor();
                void disable_float_cursor();
                void enable_jog_slide_cursor();
                void disable_jog_slide_cursor();
                void show_float_cursor();
                void hide_float_cursor();
                void show_work_cursor();
                void hide_work_cursor();

                // callbacks tags
                int motionTag;
                int floatCursorTag;

                // callbacks
                static int on_float_cursor_motion(GtkWidget *widget, GdkEventMotion *event, MultiTrackArea *o);
                static int on_jog_slide_motion(GtkWidget *widget, GdkEventMotion *event, MultiTrackArea *o);
        };

#include "Mixer.hh"
#include "Track.hh"
#include "AudioClip.hh"

#endif



