#ifndef COLORMANAGER_HH
#define COLORMANAGER_HH

#include <gtk/gtk.h>
#include <gdk/gdk.h>

#define MAX_COLORS 100

/** MustuxColorManager is a collection of constants and methods to handle colors in MustuxLib.
 *  It is being revised since many of these colors are specific to Protux and should not be here
 */
namespace MustuxColorManager
    {
    const int BLACK = 0;
    const int WHITE = 1;
    const int RED = 2;
    const int GREEN = 3;
    const int BACKGROUND_COLOR      = 10;
    const int FOREGROUND_COLOR		= 11;
    const int PLAYING_PEAK_COLOR    = 12;
    const int RECORDING_PEAK_COLOR  = 13;
    const int EMPTY_AREA_COLOR      = 14;
    const int TRACK_BACKGROUND_COLOR = 15;
    const int REGION_SELECT_SHADOW_COLOR    = 16 ;
    const int RENDERED_FILTER_NODE_COLOR    = 17 ;
    const int FROZEN_FILTER_NODE_COLOR      = 18 ;
    const int BYPASSED_FILTER_NODE_COLOR    = 19 ;
    const int CRITICAL_FILTER_NODE_COLOR    = 20;
    const int LCD_BACKGROUND_COLOR  = 21;
    const int LCD_OFF_DOT_COLOR     = 22;
    const int LCD_ON_DOT_COLOR      = 23;
    const int LCD_CRITICAL_DOT_COLOR        = 24;
    const int LCD_TEXT_COLOR                = 25;
    const int LCD_TEXT_BACKGROUND_COLOR     = 26;
    const int LCD_TEXT_CRITICAL_COLOR       = 27;
    const int LCD_SMALLTEXT_COLOR           = 28;
    const int REGION_SKIP_COLOR     = 29;
    const int TAKE_MARKER_COLOR     = 30;
    const int SPECTROGRAM_COLOR     = 31;
    const int LOCATOR_BACKGROUND_COLOR      = 32;
    const int LOCATOR_PEAK_COLOR            = 33;
    const int FILTER_TRACK_BACKGROUND_COLOR = 34;
    const int AUDIO_VIEWER_AXIS_COLOR       = 35;
    const int NO_AUDIO_COLOR        = 36;
    const int LIGHT_COLOR           = 37;
    const int DARK_COLOR            = 38;
    const int CLIP_BACKGROUND_COLOR = 39;

    /** Build the color map and holds it in a GtkWidget.
     *  Also being revised, since it should receive a component or a window
     *  as Map Holder.
     */
    void build(GtkWidget* parentWidget);

    /** Initialize the color map with several pre-defined colors
     */
    void init();

    /** Sets the foreground color of a entry in color map with values RGB
     */
    void set_fg(int colorindex, gushort r, gushort g, gushort b);

    /** Sets the background color of a entry in color map with values RGB
     */
    void set_bg(int colorindex, gushort r, gushort g, gushort b);

    /** Returns the color object from color map
     *  GdkGC* is a Gdk structure, but it is used in Mustux Application as the real color
     *  Every method that accepts a color as parameters actually is receiving a GdkGC structure.
     */
    GdkGC* get_color(int colorindex);
    };

#endif


