#include <iostream>
#include <string>
#include "MustuxDebugger.hh"
#include "MustuxComponent.hh"
#include "MustuxWindow.hh"
#include "MustuxColorManager.hh"


// constructors

MustuxComponent::MustuxComponent(MustuxWindow *newRootWindow)
    {
    PENTERCONS;
    rootWindow = newRootWindow;
    parentBox = rootWindow->get_box();
    parent = 0;
    PEXITCONS;
    }


MustuxComponent::MustuxComponent(MustuxComponent* parentComponent)
    {
    PENTERCONS;
    rootWindow = 0;
    parent = parentComponent;
    parentBox = parent->get_box();
    PEXITCONS;
    }

MustuxComponent::MustuxComponent(GtkWidget* aParentBox)
    {
    PENTERCONS;
    rootWindow = 0;
    parent = 0;
    parentBox = aParentBox;
    PEXITCONS;
    }

MustuxComponent::~MustuxComponent()
    {
    }

// general methods

int MustuxComponent::build()
    {
    build(VERTICAL,NOT_HOMOGENEOUS, RESIZEABLE, FILLABLE);
    return 1;
    }

int MustuxComponent::build(int direction)
    {
    build(direction,NOT_HOMOGENEOUS, RESIZEABLE, FILLABLE);
    return 1;
    }

int MustuxComponent::build(int direction, bool isHomogeneous)
    {
    build(direction, isHomogeneous, RESIZEABLE, FILLABLE);
    return 1;
    }

int MustuxComponent::build(int direction, bool isHomogeneous, bool isResizeable)
    {
    build(direction, isHomogeneous, isResizeable, FILLABLE);
    return 1;
    }

int MustuxComponent::build(int direction, bool isHomogeneous, bool isResizeable, bool isFillable)
    {
    PENTER;
    fillDirection = direction;

    if (fillDirection == HORIZONTAL)
        {
        //PMESG("Building Horizontal Box " << (isHomogeneous?"":"Not") << " Homogeneous");
        box = gtk_hbox_new (isHomogeneous, 0);
        }
    else
        {
        //PMESG("Building Vertical Box " << (isHomogeneous?"":"Not") << " Homogeneous");
        box = gtk_vbox_new (isHomogeneous, 0);
        }

    gtk_widget_show (box);

    if (rootWindow)
        {
        gtk_container_add (GTK_CONTAINER(parentBox), box);
        }
    else
        {
        gtk_box_pack_start(GTK_BOX(parentBox), box, isResizeable, isFillable, 0);
        }
    
    PEXIT;
    return 1;
    }

int MustuxComponent::init()
    {
    PENTER;
    PEXIT;
    return 1;
    }

int MustuxComponent::finalize()
    {
    PENTER;
    PEXIT;
    return 1;
    }

int MustuxComponent::unbuild()
    {
    PENTER;
    PEXIT;
    return 1;
    }

void MustuxComponent::set_fill_direction(int newFillDirection)
    {
    fillDirection = newFillDirection;
    }


void MustuxComponent::add_separator()
    {
    PENTER;
    if (fillDirection == HORIZONTAL)
        {
        GtkWidget* hseparator = gtk_hseparator_new ();
        gtk_widget_show (hseparator);
        gtk_box_pack_start(GTK_BOX (box), hseparator, false, false, 0);
        }
    else
        {
        GtkWidget* vseparator = gtk_vseparator_new ();
        gtk_widget_show (vseparator);
        gtk_box_pack_start(GTK_BOX (box), vseparator, false, false, 0);
        }
    PEXIT;
    }


void MustuxComponent::add_test_widget()
        {
        PENTER;
        GtkWidget* frame1 = gtk_frame_new (NULL);
        gtk_widget_show (frame1);
        gtk_box_pack_start(GTK_BOX (box), frame1, true, true, 0);

        GtkWidget* label1 = gtk_label_new ("I am a Test Widget !!!");
        gtk_widget_show (label1);
        gtk_container_add (GTK_CONTAINER (frame1), label1);
        gtk_widget_set_usize (label1, 120, 120);
        PEXIT;
        }


// get/set stuff

GtkWidget* MustuxComponent::get_box()
    {
    return box;
    }

MustuxComponent* MustuxComponent::get_parent()
    {
    return parent;
    }



//

