#ifndef DEBUGGER_HH
#define DEBUGGER_HH

//Debugging Macros

#define COLTAG_BLACK char(27) << "[0;31;40m"
#define COLTAG_RED char(27) << "[0;31;40m"
#define COLTAG_GREEN char(27) << "[0;32;40m"
#define COLTAG_YELLOW char(27) << "[0;33;40m"
#define COLTAG_BLUE char(27) << "[0;34;40m"
#define COLTAG_MAGENTA char(27) << "[0;35;40m"
#define COLTAG_CYAN char(27) << "[0;36;40m"
#define COLTAG_WHITE char(27) << "[0;37;40m"


#define PENTER if (MustuxDebugger::is_debug_on()) { \
                                MustuxDebugger::fill_tabs(); \
                                cout << COLTAG_GREEN << "ENTERING " << COLTAG_WHITE << __FILE__ << "::" << __FUNCTION__ << endl; \
                                MustuxDebugger::more_tabs(); \
                                }

#define PEXIT  if (MustuxDebugger::is_debug_on()) { \
                                MustuxDebugger::less_tabs(); \
                                MustuxDebugger::fill_tabs(); \
                                cout << COLTAG_BLUE << "LEAVING  " << COLTAG_WHITE << __FILE__ << "::" << __FUNCTION__ << endl; \
                                }

#define PENTERCONS if (MustuxDebugger::is_debug_on()) { \
                                MustuxDebugger::fill_tabs(); \
                                cout << COLTAG_GREEN << "ENTERING " << COLTAG_WHITE << __FUNCTION__  <<  COLTAG_CYAN << " (CONSTRUCTOR)" << COLTAG_WHITE << endl; \
                                MustuxDebugger::more_tabs();\
                                }

#define PEXITCONS  if (MustuxDebugger::is_debug_on()) { \
                                MustuxDebugger::less_tabs(); \
                                MustuxDebugger::fill_tabs(); \
                                cout << COLTAG_BLUE << "LEAVING  " << COLTAG_WHITE << __FUNCTION__ <<  COLTAG_CYAN<< " (CONSTRUCTOR)" << COLTAG_WHITE << endl; \
                                }

#define PENTERDES if (MustuxDebugger::is_debug_on()) { \
                                MustuxDebugger::fill_tabs(); \
                                cout << COLTAG_GREEN << "ENTERING " << COLTAG_WHITE << __FUNCTION__ <<  COLTAG_YELLOW << " (DESTRUCTOR)" << COLTAG_WHITE << endl; \
                                MustuxDebugger::more_tabs();\
                                }

#define PEXITDES  if (MustuxDebugger::is_debug_on()) { \
                                MustuxDebugger::less_tabs(); \
                                MustuxDebugger::fill_tabs(); \
                                cout << COLTAG_BLUE << "LEAVING  " << COLTAG_WHITE << __FUNCTION__ << COLTAG_YELLOW << " (DESTRUCTOR)" << COLTAG_WHITE << endl; \
                                }

#define PMARK(x) if (MustuxDebugger::is_debug_on()) { \
                                cout << COLTAG_YELLOW << "--> Reached mark " << x << COLTAG_WHITE << endl; \
                                }

#define PERROR(msg) if (MustuxDebugger::is_debug_on()) { \
                                MustuxDebugger::fill_tabs(); \
                                cerr << endl << COLTAG_RED << "*** Error in " << __PRETTY_FUNCTION__ << endl << msg << endl; \
                                }

#define PMESG(msg)  if (MustuxDebugger::is_debug_on()) { \
                                MustuxDebugger::fill_tabs(); cout << COLTAG_MAGENTA << "[ " << msg << " ]" << COLTAG_WHITE << endl; \
                                }

#define PFULLMESG(msg) if (MustuxDebugger::is_debug_on()) { \
                                MustuxDebugger::fill_tabs(); cout << COLTAG_MAGENTA << "[ " << __FILE__ << "::" << __FUNCTION__ << " - " << msg << " ]" << COLTAG_WHITE << endl; \
                                }

#define PFLOODMESG(msg) if (MustuxDebugger::is_flood_debug_on()) { \
                                          cout << "    <" << msg << ">" << endl; \
                                          }
#define PFLOODENTER     if (MustuxDebugger::is_flood_debug_on()) { \
                                          cout << "ENTERING " << __FILE__ << "::" << __FUNCTION__ << endl ; \
                                          }
#define PFLOODEXIT      if (MustuxDebugger::is_flood_debug_on()) { \
                                          cout << "LEAVING " << __FILE__ << "::" << __FUNCTION__ << endl; \
                                          }


#include <iostream>

/** MustuxDebugger is a collection of macros that makes easier the job of debugging
 *  a Mustux Application.
 *  The list of macros is :
 *  PENTER - Outputs a message when entering a method. Used in the FIRST line of a method;
 *  PEXIT  - Outputs a message when leaving a method. Used in the LAST line of a method,
 *           except if the last line is a return statement (in this case is put immediately before
 *           the return statement
 *  PMESG(message) - Outputs a message
 *  PENTERCONS - Outputs a message when entering a constructor. Similar to PENTER
 *  PEXITCONS  - Outputs a message when leaving a constructor. Similar to PEXIT
 *  PENTERDES  - Outputs a message when entering a destructor. Similar to PENTER
 *  PEXITDES   - Outputs a message when leaving a destructor. Similar to PEXIT
 *  PFLOODENTER - Similar to PENTER but recommended for methods intensively called
 *  PFLOODEXIT  - Similar to PEXIT but recommended for methods intensively called
 */

namespace MustuxDebugger
        {
        /* Used internally by MustuxDebugger. Align the output with the level of execution in a given
        * moment
        */
        void fill_tabs();

        /* Used internally by MustuxDebugger. Increase one level of execution in output messages
        * moment
        */
        void more_tabs();

        /* Used internally by MustuxDebugger. Decrease one level of execution in output messages
        */
        void less_tabs();

        void set_debug_on();
        void set_debug_off();
        void set_flood_debug_on();
        void set_flood_debug_off();

        /* Used internally by MustuxDebugger. Returns true if debugOn flag is true.
        */
        bool is_debug_on();

        /* Used internally by MustuxDebugger. Returns true if floodDebugOn flag is true.
        */
        bool is_flood_debug_on();

        }



#endif



