#include <iostream>
#include <string>
#include "MustuxColorManager.hh"
#include "MustuxInterface.hh"
#include "MustuxConfirmWindow.hh"
#include "MustuxDebugger.hh"


MustuxInterface::MustuxInterface():MustuxWindow()
    {
    PENTERCONS;
    PEXITCONS;
    }

MustuxInterface::~MustuxInterface()
    {
    PENTERDES;
    PEXITDES;
    }


int MustuxInterface::build()
    {
    PENTER;
    MustuxWindow::build(MustuxComponent::VERTICAL, MustuxComponent::NOT_HOMOGENEOUS, MustuxComponent::RESIZEABLE);

    wMenuBar = gtk_menu_bar_new ();
    gtk_widget_show (wMenuBar);
    gtk_box_pack_start (GTK_BOX (box), wMenuBar, MustuxComponent::NOT_RESIZEABLE, MustuxComponent::NOT_FILLABLE, 0);
    gtk_widget_set_usize (wMenuBar, -2, 25);

    exitMenu = add_menu("Exit");
    miExit = add_menu_item(exitMenu,"Exit");

    mainContainer =  new MustuxComponent(this);
    mainContainer->build(MustuxComponent::VERTICAL,
                         MustuxComponent::NOT_HOMOGENEOUS,
                         MustuxComponent::RESIZEABLE,
                         MustuxComponent::FILLABLE);

    lcdsContainer = new MustuxComponent(mainContainer);
    lcdsContainer->build(MustuxComponent::HORIZONTAL,
                         MustuxComponent::NOT_HOMOGENEOUS,
                         MustuxComponent::NOT_RESIZEABLE,
                         MustuxComponent::FILLABLE);
    defaultLcd = new MustuxLcd(lcdsContainer);
    defaultLcd->build(360, 100, MustuxComponent::RESIZEABLE);

    lcdsContainer->add_separator();

    projectsContainer = new MustuxNoteBook(mainContainer);
    projectsContainer->build(MustuxComponent::HORIZONTAL);

    numberProjectsOpen = 0;

    jmb = new MustuxJogMouseBoard();

    PEXIT;
    return 1;
    }


int MustuxInterface::init()
    {
    PENTER;
    MustuxWindow::init();
    MustuxColorManager::build(window);
    MustuxColorManager::init();
    mainContainer->init();

    lcdsContainer->init();
    defaultLcd->init();
    defaultLcd->draw_all_grids();
    projectsContainer->init();
    jmb->init(this);
    ML_CONNECT_CALLBACK(miExit,"activate",on_exit);
    //gtk_signal_connect (GTK_OBJECT (miExit), "activate", GTK_SIGNAL_FUNC(on_exit), this);
    PEXIT;
    return 1;
    }


int MustuxInterface::finalize()
    {
    PENTER;
    MustuxWindow::finalize();
    PEXIT;
    return 1;
    }



int MustuxInterface::unbuild()
    {
    PENTER;
    MustuxWindow::unbuild();
    PEXIT;
    return 1;
    }



void MustuxInterface::add_menu_separator(GtkWidget* menu)
    {
    GtkWidget* vseparator = gtk_menu_item_new ();
    gtk_widget_show (vseparator);
    gtk_container_add (GTK_CONTAINER (menu), vseparator);
    gtk_widget_set_sensitive (vseparator, false);
    }


GtkWidget* MustuxInterface::add_menu(string title)
    {
    PENTER;
    GtkWidget* mbit = gtk_menu_item_new_with_label (title.c_str());
    gtk_widget_show (mbit);
    gtk_container_add (GTK_CONTAINER (wMenuBar), mbit);
    GtkWidget* menu = gtk_menu_new ();
    gtk_menu_item_set_submenu (GTK_MENU_ITEM (mbit), menu);
    PEXIT;
    return menu;
    }


GtkWidget* MustuxInterface::add_menu_item(GtkWidget* menu, string item)
    {
    PENTER;
    GtkWidget* mit = gtk_menu_item_new_with_label (item.c_str());
    gtk_widget_show (mit);
    gtk_container_add (GTK_CONTAINER (menu), mit);
    PEXIT;
    return mit;
    }

void MustuxInterface::set_welcome_msg(string text)
        {
        welcomeMsg=text;
        }


void MustuxInterface::say_welcome()
        {
        defaultLcd->say_welcome(welcomeMsg);
        }


void MustuxInterface::set_app_name(string aAppName)
    {
    }


void MustuxInterface::set_app_version(string aAppName)
    {
    }


void MustuxInterface::set_app_desc(string aAppName)
    {
    }



int MustuxInterface::process_action(string actionName /*, int grabbedVal*/)
    {
    }


/*void MustuxInterface::add_project(string label)
    {
    PENTER;
    noteBook->add();
    PEXIT;
    }*/

void MustuxInterface::set_title(string an, string av, string ad)
    {
    appName = an;
    appVersion = av;
    appDesc = ad;
    MustuxWindow::set_title(appName/*+appVersion+appDesc*/);
    }


MustuxLcd* MustuxInterface::get_default_lcd()
    {
    PENTER;
    PEXIT;
    return defaultLcd;
    }

// callbacks
ML_DEFINE_CALLBACK(on_exit, MustuxInterface)
        {
        MustuxConfirmWindow* cw = new MustuxConfirmWindow();
        bool answer = cw->get_answer("Are you sure you want to quit ?");
        delete cw;
        if (answer)
                {
                stop_interaction();
                finalize();
                }
        }

/*int MustuxInterface::on_exit(GtkWidget *w, MustuxInterface* o)
    {
    }*/

// EOF

