#ifndef MUSTUXINTERFACE_HH
#define MUSTUXINTERFACE_HH

#include <gtk/gtk.h>
#include <string>

#include "MustuxWindow.hh"
#include "MustuxKernel.hh"

class MustuxJogMouseBoard;
class MustuxLcd;
class MustuxComponent;
class MustuxNoteBook;

/** MustuxInterface is a simple Interface with a menu (just one option : exit)
 *  a lcdContainer with one single LCD (for messages outputs), and methods to
 *  set several atributes in a interface
 */
class MustuxInterface  : public MustuxWindow
    {
    public : 
        MustuxInterface();
        virtual ~MustuxInterface();
        /** build the interface and all its internal widgets/components
         */
        int build();
        
        /** Initialize internal components and attach events to callbacks
         */
        int init();
        
        /** deattach events from callbacks and preares to unbuild internal components
         */
        int finalize();
        
        /** unbuild internal widgets/components and de-allocate memory
         */
        int unbuild();
        
        /** returns a referente to the standard LCD used for messages output
         */
        MustuxLcd* get_default_lcd();

        /** sets the title of the interface (in the upper bar)
         */
        void set_title(string an, string av, string ad);

        /** sets the application name for this interface
         */
        void set_app_name(string aAppName);

        /** sets the application version
         */
        void set_app_version(string aAppName);

        /** sets the application description
         */
        void set_app_desc(string aAppName);

        /** return the application name
         */
        string get_app_name();
        
        /** returns the application version
         */
        string get_app_version();

        /** return the application description
         */
        string get_app_desc();

        /** This is an abstract method that must be implement by
         *  derivated (customized) interfaces. It handles JMB actions
         *  and call apropriated handles for each one
         */
        virtual int process_action(string actionName/*, int grabbedVal*/);

    protected:
        MustuxJogMouseBoard* jmb;
        MustuxComponent* mainContainer;
        MustuxComponent* lcdsContainer;
        MustuxNoteBook* projectsContainer;
        MustuxLcd* defaultLcd;
        string appName;
        string appVersion;
        string appDesc;
        string welcomeMsg;

        void set_welcome_msg(string text);
        void say_welcome();

        GtkWidget* wMenuBar;
        GtkWidget* exitMenu;
        GtkWidget* miExit;
        int numberProjectsOpen;

        GtkWidget* add_menu(string title);
        GtkWidget* add_menu_item(GtkWidget* menu, string item);
        void add_menu_separator(GtkWidget* menu);

        // callbacks
        ML_DECLARE_CALLBACK(on_exit);
        // TODO : transform next two callbacks into to ML_DECLARE_CALLBACK...
        static int on_switch_project(GtkNotebook *notebook, GtkNotebookPage *page, int page_num, MustuxInterface* i);
        static int on_configure_projects_area(GtkWidget *widget, GdkEventConfigure *event, MustuxInterface* i);
        };

#include "MustuxLcd.hh"
#include "MustuxComponent.hh"
#include "MustuxJogMouseBoard.hh"
#include "MustuxNoteBook.hh"

#endif

