#include <string>
#include <cstdio>
#include <gdk/gdkkeysyms.h>
#include "MustuxJogMouseBoard.hh"
#include "MustuxJogMouseBoardEvent.hh"
#include "MustuxDebugger.hh"
//FOR LATER#include "SettingsManager.hh"




MustuxJogMouseBoard::MustuxJogMouseBoard():MustuxEngine()
        {
        PENTERCONS;
        PEXITCONS;
        }

MustuxJogMouseBoard::~MustuxJogMouseBoard()
        {
        PENTERDES;
        PEXITDES;
        }


int MustuxJogMouseBoard::init(MustuxInterface *aMustuxInterface)
        {
        PENTER;
        MustuxEngine::init(aMustuxInterface);
        interface = aMustuxInterface;
        activate();
        clearOutputTag=0;
        reset();
        init_map();
        return 1;
        PEXIT;
        }


void MustuxJogMouseBoard::activate()
        {
        PENTER;
        print("JMB ACTIVE", 5);
        keySnooperTag = gtk_key_snooper_install ((GtkKeySnoopFunc) catch_event, this);
        isFirstPress=true;
        PEXIT;
        }


void MustuxJogMouseBoard::suspend()
        {
        PENTER;
        gtk_key_snooper_remove (keySnooperTag);
        print("JMB SUSPENDED", 5);
        PEXIT;
        }


int MustuxJogMouseBoard::finalize()
        {
        PENTER;
        MustuxEngine::finalize();
        PEXIT;
        return 1;
        }


// ----------------------- JMB ENGINE : EVENT LEVEL HANDLING ---------------------

// reset the event stack and the press 'stack' (not exactly a stack)
void MustuxJogMouseBoard::reset()
        {
        PFLOODENTER;
        isFirstPress = true;
        isHolding = false;
        isTimerRunning = false;
        isKeyPressLocked = false;
        stackIndex = 0;
        keyPressCounter = 0;
        holdTime=0;
        press1_k1 = 0;
        press1_k2 = 0;
        press2_k1 = 0;
        press2_k2 = 0;
        wholeAction = -1;

        for (int i=0; i < STACK_SIZE; i++)
                {
                eventType[i] = 0;
                eventStack[i] = 0;
                time[i] = 0;
                }
        PFLOODEXIT;
        }


// Everthing starts here. Catch event takes anything happen in the keyborad and pushes it
// into a stack.
int MustuxJogMouseBoard::catch_event(GtkWidget *grab_widget, GdkEventKey *event, MustuxJogMouseBoard *j)
        {
        PFLOODENTER;
        event->keyval=gdk_keyval_to_upper(event->keyval);
        if (event->type == GDK_KEY_PRESS)
                {
                if (!j->isKeyPressLocked)
                        {
                        if (j->keyPressCounter < 2)
                                {
                                j->keyPressCounter++;
                                j->push_event(event->type, event->keyval, event->time);
                                j->start_hold_timer(); // launch a temporizer
                                }
                        else
                                {
                                j->isKeyPressLocked = true;
                                }
                        }
                }
        if (event->type == GDK_KEY_RELEASE)
                {
                if (!j->is_fake(event->keyval))
                        {
                        j->push_event(event->type, event->keyval, event->time);
                        j->release_checker();
                        }
                }
        PFLOODEXIT;
        return 1;
        }



// this is a intermediate step to push. Only non-fake events are allowed to be pushed
bool MustuxJogMouseBoard::is_fake(int keyval)
        {
        // A fake event is an event which is generated by a ignored GDK_KEY_PRESS
        return (
                           (keyval!=eventStack[0])
                        && (keyval!=eventStack[1])
                        && (keyval!=eventStack[2])
                        && (keyval!=eventStack[3])
                );
        }


// This pushed an event to the stack
void MustuxJogMouseBoard::push_event(  int aType,  int anEvent,  int aTime )
        {
        PFLOODENTER;
        if (stackIndex < STACK_SIZE)
                {
                eventType[stackIndex] = aType;
                eventStack[stackIndex] = anEvent;
                time[stackIndex] = aTime;
                stackIndex++;
                }
        PFLOODEXIT
        }


// This is the one who consolidate a press. It can detect
// if the press is K or KK, and also calls the push_press handler
void MustuxJogMouseBoard::release_checker()
        {
        PFLOODENTER;
        if (isHolding)
                {
                finish_hold();
                return;
                }
        // If it is not fake, then lets take a look in the pairs
        // Remember that eventType[0]==GDK_KEY_PRESS always
        if (eventType[1]==GDK_KEY_RELEASE)
                {
                // action 0 and action 1 are a pair because action 1 must be the release of 0
                // since the action 1 was a release, then the event is finished
                // Now we must push_press, but only if it is a release very fast (that
                // didnt cause a HOLD.
                if (!isHolding)
                        push_press(eventStack[0],0);
                PFLOODEXIT;
                return;
                }
        // action 1 is ALSO a GDK_KEY_PRESS. So we have a Double_Key
        // Now we dont know if we are in action 2 (stackIndex==2)
        // or in the action 3 (stackIndex==3)
        if ((stackIndex-1)==2)
                {
                // So we are processing the action 2, which is a release
                // So action 2 can be release of action 0 or 1 , so..
                if (eventStack[2]==eventStack[0])
                       pairOf2 = 0;
                else
                       pairOf2 = 1;
                // the event is not finished yet but it must be avoided
                // any reentrance (any new GDK_KEY_PRESS)
                isKeyPressLocked = true;
                PFLOODEXIT;
                return;
                }
        // we are in the action 3 . the last!
        // the event is finished , surely. But we must check the pairs
        // we already know who action 2 is pair of. So ...
        if (pairOf2 == 0)
                pairOf3 = 1;
        else
                pairOf3 = 0;
        // The event is finished, and we know the pairs. So..
        // I must push_press, but only if it is a release very fast (that
        // didnt cause a HOLD.
        if (!isHolding)
                push_press(eventStack[0],eventStack[1]);
        PFLOODEXIT;
        }


void MustuxJogMouseBoard::start_hold_timer()
        {
        PFLOODENTER;
        if (!isTimerRunning)
                {
                holdTime = 0;
                isTimerRunning = true;
                holdTimerTag = gtk_timeout_add( 5, hold_timer, this );
                }
        PFLOODEXIT;
        }


int MustuxJogMouseBoard::hold_timer(gpointer obj)
        {
        PFLOODENTER;
        MustuxJogMouseBoard *j = (MustuxJogMouseBoard*) obj;
        j->holdTime++;
        if (j->holdTime > 10)
                {
                PFLOODMESG("Hold timer timeout. Dispatching hold");
                j->isHolding = true;
                gtk_timeout_remove(j->holdTimerTag);
                j->dispatch_hold();
                }
        PFLOODEXIT;
        return 1;
        }


// ----------------------------- JMB ENGINE : PRESS LEVEL HANDLING -------------------------


// This guy pushes a press (the first and the second)
void MustuxJogMouseBoard::push_press(int k1,int k2)
        {
        PFLOODENTER;
        PFLOODMESG("Pushing press :: k1=" << k1 << " k2=" << k2);
        print("", 5);
        gtk_timeout_remove(holdTimerTag);
        if (isFirstPress)
                {
                // this is the first press
                PFLOODMESG("First press detected !");
                // first try to find some action like k1k2
                press1_k1 = k1;
                press1_k2 = k2;
                int action = identify_first_press();
                PFLOODMESG("Got " << action << " from identify_first_press()");
                if (action < 0)
                        {
                        // Action not identified. Maybe is part of a Double press action. so...
                        give_a_chance_for_second_press();
                        PFLOODEXIT;
                        return;
                        }
                // action exists !! now must check if is not an immediate action
                if (!map[action].is_immediate())
                        {
                        PFLOODMESG("Although this could be an SINGLE PRESS Action, it is not protected, so...");
                        // action is not an immediate action, so...
                        give_a_chance_for_second_press();
                        PFLOODEXIT;
                        return;
                        }

                // Action exists AND it is a immediate action. So
                // forces it to be a SinglePressAction
                PFLOODMESG("This is immediate action. Giving immediate response");
                //isDoublePress = false;
                dispatch_action(action);
                conclusion();
                }
        else
                {
                gtk_timeout_remove(secondChanceTag);
                press2_k1 = k1;
                press2_k2 = k2;
                if (press2_k1!=0)
                        {
                        // this is the second press
                        PMESG("Second press detected !");
                        }

                // try to complement the first press.
                wholeAction = complement_first_press();
                PFLOODMESG("Got " << wholeAction << " from complement_first_press()");
                if (wholeAction >= 0)
                        {
                        PFLOODMESG("Action " << wholeAction << " recognized. Dispatching...");
                        dispatch_action(wholeAction);
                        }
                else
                        {
                        print("UNKNOWN ACTION", 1);
                        }

                conclusion();
                }
        PFLOODEXIT;
        }





int MustuxJogMouseBoard::identify_first_press()
        {
        PFLOODENTER;
        press1Type = 0;
        // First we need to know the first press action type.
        if (press1_k2==0) // {K}
                {
                if (!isHolding)
                        {
                        //PMESG("Detected <K>");
                        press1Type = FKEY; // <K>
                        }
                else
                        {
                        //PMESG("Detected [K]");
                        press1Type = HKEY; // [K]
                        }
                }
        else // {KK}
                {
                if (!isHolding)
                        press1Type = FKEY2; // <KK>
                else
                        press1Type = HKEY2; // [KK]
                }

        // Press 1 Type identified .
        // Now we ask : is there any action like {k1k2}/{0 0} ?
        // PMESG("Now searching for a match SINGLE PRESS action");
        for (int i=0; i<mapSize; i++)
                        {
                        int ap1k1 = map[i].get_press1_key1();
                        int ap1k2 = map[i].get_press1_key2();
                        int ap2k1 = map[i].get_press2_key1();
                        int ap2k2 = map[i].get_press2_key2();

                        // PMESG("COMPARING " <<  ap1k1     << "," << ap1k2     << "," << ap2k1 << "," << ap2k2 <<   WITH "    <<  press1_k1 << "," << press1_k2 << ",0,0")

                        if (map[i].get_action_type() != press1Type ) continue;


                        if (
                                (
                                 ((ap1k1==press1_k1) && (ap1k2==press1_k2))
                                 ||
                                 ((ap1k1==press1_k2) && (ap1k2==press1_k1))
                                )
                                &&
                                ( ap2k1 == 0 )
                                &&
                                ( ap2k2 == 0 )
                           )
                                {
                                // PMESG("Found a match in i=" << i);
                                // 'i' is a candidate for first press
                                PFLOODEXIT;
                                return i;
                                }
                        }
        // No candidate found.
        PFLOODEXIT;
        return -1;
        }





int MustuxJogMouseBoard::complement_first_press()
        {
        PFLOODENTER;
        PFLOODMESG("Adding the pair k1=" << press2_k1 << " k2=" << press2_k2 << " to press1_k1=" << press1_k1 << ",press1_k2=" << press1_k2);
        int wholeActionType;
        if (press2_k1!=0)
                {
                if (!isHolding)
                        {
                        if (press1_k2==0) // first press is <K> (I know that its not [K] because if it was I'd never reach complement_first_press())
                                {
                                if (press2_k2==0)
                                        if (press1_k1 == press2_k1)
                                                {
                                                PFLOODMESG("Whole action is a <<K>>");
                                                wholeActionType = D_FKEY;  // <<K>>
                                                }
                                        else
                                                {
                                                PFLOODMESG("Whole action is a >K>K");
                                                wholeActionType = S_FKEY_FKEY; // >K>K
                                                }
                                else
                                        {
                                        PFLOODMESG("Whole action is a >K>KK");
                                        wholeActionType = S_FKEY_FKEY2;  // >K>KK
                                        }
                                }

                        else
                                {
                                if (press2_k2==0)
                                        {
                                        PFLOODMESG("Whole action is a >KK>K");
                                        wholeActionType = S_FKEY2_FKEY;  // >KK>K
                                        }
                                else
                                        {
                                        PFLOODMESG("Whole action is a >KK>KK");
                                        wholeActionType = S_FKEY2_FKEY2; // >KK>KK
                                        }
                                }
                        }
                else
                        {
                        if (press1_k2==0) // first press is <K> (I know that its not [K] because if it was I'd never reach complement_first_press())
                                {
                                if (press2_k2==0)
                                        if (press1_k1 == press2_k1)
                                                {
                                                PFLOODMESG("Whole action is a <[K]>");
                                                wholeActionType = FHKEY;  // <[K]>
                                                }
                                        else
                                                {
                                                PFLOODMESG("Whole action is a >K[K]");
                                                wholeActionType = S_FKEY_HKEY;  // >K[K]
                                                }
                                else
                                        {
                                        PFLOODMESG("Whole action is a >K[KK]");
                                        wholeActionType = S_FKEY_HKEY2;  // >K[KK]
                                        }
                                }

                        else
                                {
                                if (press2_k2==0)
                                        {
                                        PFLOODMESG("Whole action is a KK>K");
                                        wholeActionType = S_FKEY2_FKEY;  // KK>K
                                        }
                                else
                                        {
                                        PFLOODMESG("Whole action is a >KK>KK");
                                        wholeActionType = S_FKEY2_FKEY2; // >KK>KK
                                        }
                                }
                        }
                }
        else
                {
                PFLOODMESG("Second Press is totally null. Assuming wholeActionType = press1Type (" << press1Type << ")");
                wholeActionType = press1Type;
                }
        // whole action type identified .

        PFLOODMESG("Now searching for a match action...");
        for (int i=0; i<mapSize; i++)
                        {
                        if ( map[i].get_action_type() != wholeActionType ) continue;
                        int ap1k1 = map[i].get_press1_key1();
                        int ap1k2 = map[i].get_press1_key2();
                        int ap2k1 = map[i].get_press2_key1();
                        int ap2k2 = map[i].get_press2_key2();
                        PFLOODMESG("COMPARING " <<  ap1k1     << "," << ap1k2     << "," << ap2k1 << "," << ap2k2 <<  " WITH "    <<  press1_k1 << "," << press1_k2 << ",0,0")
                        if (
                                (
                                 ((ap1k1==press1_k1) && (ap1k2==press1_k2))
                                 ||
                                 ((ap1k1==press1_k2) && (ap1k2==press1_k1))
                                )
                                &&
                                (
                                 ((ap2k1==press2_k1) && (ap2k2==press2_k2))
                                 ||
                                 ((ap2k1==press2_k2) && (ap2k2==press2_k1))
                                )
                           )
                                {
                                PFLOODMESG("Found a match in i=" << i);
                                // 'i' is a candidate the whole action
                                PFLOODEXIT;
                                return i;
                                }
                        }
        // No candidate found.
        PFLOODEXIT;
        return -1;
        }


void MustuxJogMouseBoard::give_a_chance_for_second_press()
        {
        PENTER;
        static int quitTime = 200;
        PFLOODMESG("Waiting " <<  quitTime << "ms for second press...");
        secondChanceTag = gtk_timeout_add( quitTime, quit_second_chance, this );
        isFirstPress=false;
        PFLOODEXIT;
        }



int MustuxJogMouseBoard::quit_second_chance(gpointer obj)
        {
        PFLOODENTER;
        MustuxJogMouseBoard *j = (MustuxJogMouseBoard*) obj;
        gtk_timeout_remove(j->secondChanceTag);
        PFLOODMESG("I'm sorry, you lost your chance for second press...");
        j->push_press(0 , 0); // no second press performed, so I am adding a pair of Zeros as second press and keep going...
        PFLOODEXIT;
        return 1;
        }





// ----------------------------- JMB ENGINE : ACTION LEVEL HANDLING -------------------------
void MustuxJogMouseBoard::dispatch_action(int action)
        {
        PFLOODENTER;
        print(map[action].actionName, 1);
        PMESG("Processing Action " << action );
        interface->process_action(map[action].actionName);
        PFLOODEXIT;
        }



// This is called by
void MustuxJogMouseBoard::dispatch_hold()
        {
        PFLOODENTER;
        if (clearOutputTag!=0)
                {
                gtk_timeout_remove(clearOutputTag);
                clearOutputTag = 0;
                isHoldingOutput=false;
                }
        print("", 5);

        wholeAction = -1;
        if (isFirstPress)
                {
                press1_k1 = eventStack[0];
                press1_k2 = eventStack[1];
                wholeAction = identify_first_press(); // I can consider first press the last because there is nothing after a []
                }
        else
                {
                press2_k1 = eventStack[0];
                press2_k2 = eventStack[1];
                wholeAction = complement_first_press();
                }

        if (wholeAction>=0)
                {
                MustuxJogMouseBoardEvent jmbe = map[wholeAction];
                print(jmbe.actionName, 1);
                if ((jmbe.use_X_axis()) && (jmbe.use_Y_axis()))
                        print("MOVE MOUSE U/D/L/R", 2);
                else if (jmbe.use_X_axis())
                        print("MOVE MOUSE L/R", 2);
                else if(jmbe.use_Y_axis())
                        print("MOVE MOUSE U/D", 2);
                PFLOODMESG("Processing hold action " << wholeAction );
                interface->process_action(jmbe.actionName);
                }
        else
                print("UNKNOWN ACTION", 1);
        PFLOODEXIT;
        // note that we dont call conclusion here :)
        }



void MustuxJogMouseBoard::finish_hold()
        {
        PFLOODENTER;
        print("", 2);
        PFLOODMESG("Finishing hold action " << wholeAction);
        interface->process_action(map[wholeAction].actionName);
        conclusion();
        PFLOODEXIT;
        }


void MustuxJogMouseBoard::conclusion()
    {
    PFLOODENTER;
    print("ACTION FINISHED", 5);
    reset();
    hold_output();
    PFLOODEXIT;
    }





void MustuxJogMouseBoard::hold_output()
        {
        PFLOODENTER;
        if (!isHoldingOutput)
                {
                clearOutputTag = gtk_timeout_add( 2000, clear_output, this );
                PFLOODMESG("holding output TAG = " << clearOutputTag);
                isHoldingOutput=true;
                }
        PFLOODEXIT;
        }



int MustuxJogMouseBoard::init_map()
        {
        #define SKIP do fread(&c, 1, 1, mapFile); while (((c==0x20) || (c==0x09)) && (!feof(mapFile)));
        PFLOODENTER;
        string mapFilename = "jmb.map";// FOR LATER SettingsManager::get_property("JMB_MAP_FILE");
        FILE* mapFile = fopen(mapFilename.c_str(), "r");
        if (mapFile)
                {
                PMESG("INITIALIZING JMB MAP ... ");
                char c=0;
                int index=0;
                while (!feof(mapFile))
                        {
                        bool useX,useY,grabv;
                        int actionTypeCode, nKeys;
                        int j;
                        char skey[16];
                        int keyCode[4];
                        char actionType[32];
                        char label[256];

                        // ignore everything until next "
                        do
                                fread(&c, 1, 1, mapFile);
                        while ((c!=0x22) && (!feof(mapFile)));
                        // first " found. everything until next " is the label
                        j=0;
                        do
                                {
                                fread(&c, 1, 1, mapFile);
                                label[j++]=c;
                                }
                        while ((c!=0x22) && (!feof(mapFile)));
                        label[--j]=0;
                        if (feof(mapFile)) break;
                        SKIP;

                        // get the type of action
                        j=0;
                        while ((c!=0x20) && (c!=0x09) && (!feof(mapFile)))
                                {
                                actionType[j++]=c;
                                fread(&c, 1, 1, mapFile);
                                }
                        actionType[j]=0;

                        if      ( strcmp(actionType,"<K>")    == 0 )  { actionTypeCode = FKEY;          nKeys = 1; }
                        else if ( strcmp(actionType,"<KK>")   == 0 )  { actionTypeCode = FKEY2;         nKeys = 2; }
                        else if ( strcmp(actionType,"[K]")    == 0 )  { actionTypeCode = HKEY;          nKeys = 1; }
                        else if ( strcmp(actionType,"[KK]")   == 0 )  { actionTypeCode = HKEY2;         nKeys = 2; }
                        else if ( strcmp(actionType,"<<K>>")  == 0 )  { actionTypeCode = D_FKEY;        nKeys = 1; }
                        else if ( strcmp(actionType,"<<KK>>") == 0 )  { actionTypeCode = D_FKEY2;       nKeys = 2; }
                        else if ( strcmp(actionType,"<[K]>")  == 0 )  { actionTypeCode = FHKEY;         nKeys = 1; }
                        else if ( strcmp(actionType,"<[KK]>") == 0 )  { actionTypeCode = FHKEY2;        nKeys = 2; }
                        else if ( strcmp(actionType,">K>K")   == 0 )  { actionTypeCode = S_FKEY_FKEY;   nKeys = 2; }
                        else if ( strcmp(actionType,">K>KK")  == 0 )  { actionTypeCode = S_FKEY_FKEY2;  nKeys = 3; }
                        else if ( strcmp(actionType,">KK>K")  == 0 )  { actionTypeCode = S_FKEY2_FKEY;  nKeys = 3; }
                        else if ( strcmp(actionType,">KK>KK") == 0 )  { actionTypeCode = S_FKEY2_FKEY2; nKeys = 4; }
                        else if ( strcmp(actionType,">K[K]")  == 0 )  { actionTypeCode = S_FKEY_HKEY;   nKeys = 2; }
                        else if ( strcmp(actionType,">K[KK]") == 0 )  { actionTypeCode = S_FKEY_HKEY2;  nKeys = 3; }
                        else if ( strcmp(actionType,">KK[K]") == 0 )  { actionTypeCode = S_FKEY2_HKEY;  nKeys = 3; }
                        else if ( strcmp(actionType,">KK[KK]")== 0 )  { actionTypeCode = S_FKEY2_HKEY2; nKeys = 4; }
                        if (feof(mapFile)) break;
                        SKIP;

                        // get use_X
                        while ((c!=0x20) && (c!=0x09) && (!feof(mapFile)))
                                {
                                useX = (c!=0x30);
                                fread(&c, 1, 1, mapFile);
                                }
                        if (feof(mapFile)) break;
                        SKIP;

                        // get use_Y
                        while ((c!=0x20) && (c!=0x09) && (!feof(mapFile)))
                                {
                                useY=(c!=0x30);
                                fread(&c, 1, 1, mapFile);
                                }
                        if (feof(mapFile)) break;
                        SKIP;

                        // get grabVal - DEPRECATED - Grabbing values will not be handled outside JMB scope
                        while ((c!=0x20) && (c!=0x09) && (!feof(mapFile)))
                                {
                                grabv=(c!=0x30);
                                fread(&c, 1, 1, mapFile);
                                }
                        if (feof(mapFile)) break;
                        SKIP;

                        // get nKeys hex codes
                        for (int nk=0; nk<4; nk++) keyCode[nk]=0;
                        for (int nk=0; nk<nKeys; nk++)
                                {
                                j=0;
                                while ((c!=0x20) && (c!=0x09) && (!feof(mapFile)))
                                        {
                                        skey[j++]=c;
                                        fread(&c, 1, 1, mapFile);
                                        }
                                skey[j]=0;
                                int keycode=strtol(skey,0,16);
                                keyCode[nk]=keycode;
                                SKIP;
                                }
                        if (feof(mapFile)) break;
                        PFLOODMESG("ADDED action: " << label << " Type=" << actionType << " useX=" << useX << "  useY=" << useY << "  key1=" << keyCode[0] << " key2=" << keyCode[1] << "  key3=" << keyCode[2] << " key4=" << keyCode[3]);

                        // Fix the keyCode positions
                        if      (( actionTypeCode == D_FKEY  )     ||
                                 ( actionTypeCode == FHKEY  ))        { keyCode[2]=keyCode[0]; }
                        else if (( actionTypeCode == S_FKEY_FKEY ) ||
                                 ( actionTypeCode == S_FKEY_HKEY ))   { keyCode[2]=keyCode[1]; keyCode[1]=0; }
                        else if (( actionTypeCode == D_FKEY2 )     ||
                                 ( actionTypeCode == FHKEY  ))        { keyCode[2]=keyCode[0]; keyCode[3]=keyCode[1]; }
                        map[index++].set(label, actionTypeCode, keyCode[0], keyCode[1], keyCode[2], keyCode[3], useX, useY, false);
                        }
                fclose (mapFile);
                mapSize=index;
                PFLOODMESG("Optimizing map for best performance ...");
                int optimizedActions=0;
                for (int i=0; i<mapSize; i++)
                        {
                        int c1A=map[i].get_press1_key1();
                        int c2A=map[i].get_press1_key2();

                        if ( (map[i].get_action_type()==FKEY) )
                                {
                                bool alone = true;
                                for (int j = 0; j < mapSize; j++)
                                        {
                                        if (j==i) continue;
                                        int tt    = map[j].get_action_type();
                                        int t1A   = map[j].get_press1_key1();
                                        if  (
                                              (t1A==c1A)  &&
                                              (
                                                (tt==D_FKEY)       ||
                                                (tt==FHKEY)        ||
                                                (tt==S_FKEY_FKEY)  ||
                                                (tt==S_FKEY_FKEY2) ||
                                                (tt==S_FKEY_HKEY)  ||
                                                (tt==S_FKEY_HKEY2)
                                              )
                                             )
                                             alone=false;
                                        }
                                if (alone)
                                        {
                                        PFLOODMESG("Setting <K> action " << map[i].actionName << " for instantaneous response");
                                        map[i].set_immediate(true);
                                        optimizedActions++;
                                        }
                                else
                                        map[i].set_immediate(false);

                                }
                        else if ((map[i].get_action_type()==FKEY2))
                                {
                                bool alone = true;
                                for (int j = 0; j < mapSize; j++)
                                        {
                                        if (j==i) continue;
                                        int tt    = map[j].get_action_type();
                                        int t1A   = map[j].get_press1_key1();
                                        int t2A   = map[j].get_press1_key2();
                                        if (
                                            ((t1A==c1A) && (t2A==c2A)) &&
                                            (
                                              (tt==HKEY)         ||
                                              (tt==D_FKEY2)      ||
                                              (tt==S_FKEY2_FKEY) ||
                                              (tt==S_FKEY2_FKEY2)||
                                              (tt==S_FKEY2_HKEY) ||
                                              (tt==S_FKEY2_HKEY2)
                                            )
                                           )
                                           alone=false;
                                        }
                                if (alone)
                                        {
                                        PFLOODMESG("Setting <KK> action " << map[i].actionName << " for instantaneous response");
                                        map[i].set_immediate(true);
                                        optimizedActions++;
                                        }
                                else
                                        map[i].set_immediate(false);
                                }
                        }

                PMESG("JMB MAP INITIALIZED! " << mapSize << " actions registered ( " << optimizedActions << " optimized) .");
                }
        else
                {
                PERROR("Cannot open Jmb.map");
                }
        #undef SKIP
        PFLOODEXIT;
        return 1;
        }






// callbacks



int MustuxJogMouseBoard::clear_output(gpointer obj)
        {
        PFLOODENTER;
        MustuxJogMouseBoard *j = (MustuxJogMouseBoard*) obj;
        if ((j->isHoldingOutput))
                {
                j->output->clear();
                gtk_timeout_remove(j->clearOutputTag);
                j->clearOutputTag=0;
                j->isHoldingOutput=false;
                j->output->draw_all_grids();
                j->print("PROTUX JMB ACTIVE", 5);
                }
        PFLOODEXIT;
        return 1;
        }

// eof


