#include <string>
#include <iostream>
#include "LcdIcons.hh"
#include "LcdDigits.hh"
#include "MustuxLcd.hh"
#include "MustuxColorManager.hh"
#include "MustuxDebugger.hh"
//#include "SettingsManager.hh" // FOR LATER


MustuxLcd::MustuxLcd(MustuxComponent* parent):MustuxPane(parent)
    {
    PENTERCONS;
    PEXITCONS;
    }

MustuxLcd::~MustuxLcd()
    {
    PENTERDES;
    PEXITDES;
    }


int MustuxLcd::build(int width, int height, bool isResizeable)
    {
    PENTER;
    MustuxPane::build(width, height, isResizeable);
    isGridLcd = false;
    maxDigits = 40;
    PEXIT;
    return 1;
    }


int MustuxLcd::init()
    {
    PENTER;
    MustuxPane::init();
    clear();
    PEXIT;
    return 1;
    }


int MustuxLcd::finalize()
    {
    MustuxPane::finalize();
    return 1;
    }


int MustuxLcd::unbuild()
    {
    MustuxPane::unbuild();
    return 1;
    }


void MustuxLcd::recreate()
    {
    PFLOODENTER;
    maxDigits = (int) ((get_width() - 10) / 12);
    clear();
    if (isGridLcd)
        draw_all_grids();
    update();
    PFLOODEXIT;
    }


// draw methods

void MustuxLcd::draw_background_grid()
    {
    PENTER;
    int gridxpoints = (get_width()-10)  / DOTHSPACE;
    int gridypoints = (get_height()-10) / DOTVSPACE;
    for (int i=0; i<gridxpoints; i++)
            for (int j=0; j<gridypoints; j++)
                    gdk_draw_rectangle(drawPane, MustuxColorManager::get_color(MustuxColorManager::LCD_OFF_DOT_COLOR) , true,
                                    5 + i * DOTHSPACE,
                                    5 + j * DOTVSPACE,
                                    1,1 );
    PEXIT;
    }


void MustuxLcd::draw_icon(int index)
    {
    PFLOODENTER;
    static GdkGC* c1 = MustuxColorManager::get_color(MustuxColorManager::LCD_ON_DOT_COLOR);
    static GdkGC* c2 = MustuxColorManager::get_color(MustuxColorManager::LCD_CRITICAL_DOT_COLOR);
    char c;
    for (int a=0; a<15; a++)
            for (int b=0; b<15; b++)
                    {
                    c = icon_map[index][b][a];
                    if (c=='1')
                            gdk_draw_rectangle (storePane, c1, true,
                                            10+a*DOTHSPACE,
                                            10+b*DOTVSPACE,
                                            DOTSIZE, DOTSIZE);
                    if (c=='2')
                            gdk_draw_rectangle (storePane, c2 , true,
                                            10+a*DOTHSPACE,
                                            10+b*DOTVSPACE,
                                            DOTSIZE, DOTSIZE);
                    }
    update();
    PFLOODEXIT;
    }




void MustuxLcd::clear_line(int line)
    {
    PFLOODENTER;
    gdk_draw_rectangle (storePane, MustuxColorManager::get_color(MustuxColorManager::LCD_BACKGROUND_COLOR), true,
            0, 9 + (line-1)*DIGITVSPACE , get_width(),  DIGITVSPACE);
    update(0, 9 + (line-1)*DIGITVSPACE , get_width(),  DIGITVSPACE);
    PFLOODEXIT;
    }


void MustuxLcd::draw_all_grids()
    {
    PFLOODENTER;
    isGridLcd = true;
    for (int i=1; i<6; i++)
                    draw_grid(i);
    PFLOODEXIT;
    }


void MustuxLcd::draw_grid(int line)
    {
    PFLOODENTER;
    for (int i=0; i<maxDigits; i++)
            for (int j=0; j<5; j++)
                    for (int k=0; k<8; k++)
                            gdk_draw_rectangle(storePane, MustuxColorManager::get_color(MustuxColorManager::LCD_OFF_DOT_COLOR), true,
                                            GRID_MARGIN_LEFT+i*DIGITHSPACE+(j*2),
                                            GRID_MARGIN_UP+(line-1)*DIGITVSPACE+(k*2),1,1);

    update(0,GRID_MARGIN_UP+(line-1)*DIGITVSPACE,-1,DIGITVSPACE);
    PFLOODEXIT;
    }


void MustuxLcd::draw_digit(int index, int col,  int line)
    {
    PFLOODENTER;
    for (int a=0; a<5; a++)
        for (int b=0; b<8; b++)
            {
            if (digit[index][b][a]=='1')
                gdk_draw_rectangle (storePane, MustuxColorManager::get_color(MustuxColorManager::LCD_ON_DOT_COLOR), true,
                                GRID_MARGIN_LEFT+((col-1)*DIGITHSPACE)+(a*2),
                                GRID_MARGIN_UP-2+((line-1)*DIGITVSPACE)+(b*2),
                                DOTSIZE, DOTSIZE);
            }
    update(GRID_MARGIN_LEFT+((col-1)*DIGITHSPACE)+8,
        GRID_MARGIN_UP-1+((line-1)*DIGITVSPACE)+14,
        DIGITHSPACE, DIGITVSPACE);
    PFLOODEXIT;
    }


void MustuxLcd::print (string s, int line)
    {
    PFLOODENTER;
    print(s, 1, line, true);
    PFLOODEXIT;
    }


void MustuxLcd::print (string s, int col, int line)
    {
    PFLOODENTER;
    print (s, col, line, true);
    PFLOODEXIT;
    }


void MustuxLcd::print (string s, int line, bool clean)
    {
    PFLOODENTER;
    print (s, 1, line, clean);
    PFLOODEXIT;
    }


void MustuxLcd::print (string s, int col, int line, bool clean)
    {
    PFLOODENTER;
    const char* cs = s.c_str();
    if (clean)
            {
            clear_line(line);
            draw_grid(line);
            }
    int i=0;
    while (cs[i])
            {
            draw_digit(cs[i]-32, i+col, line);
            i++;
            }
    update();
    PFLOODEXIT;
    }


void MustuxLcd::clear()
    {
    PFLOODENTER;
    MustuxPane::clear(MustuxColorManager::LCD_BACKGROUND_COLOR);
    PFLOODEXIT;
    }


int MustuxLcd::get_max_digits() const
    {
    return maxDigits;
    }



// ICON STUFF


int MustuxLcd::start_blinking_icon(int icon)
    {
    iconToBlink = icon;
    isIconBlinking = true;
    iconBlinkTag = gtk_timeout_add( 200, blink_icon, this );
    return 1;
    }

int MustuxLcd::stop_blinking_icon()
    {
    clear();
    draw_background_grid();
    draw_icon(PAUSE_ICON);
    gtk_timeout_remove(iconBlinkTag);
    return 1;
    }


// Welcome stuff

void MustuxLcd::say_welcome( string text )
    {
    welcomeText = text;
    welcomeIndex = 0;
    while (gtk_events_pending())
        gtk_main_iteration();
    welcomeTag = gtk_timeout_add( 40, scroll_welcome, this );
    }


int MustuxLcd::scroll_welcome(gpointer obj)
    {
    MustuxLcd *o = (MustuxLcd*) obj;
    string s;
    int md = o->get_max_digits();
    if (o->welcomeIndex < (md+20))
        {
        s = o->welcomeText.substr(0, o->welcomeIndex);
        if ( o->welcomeIndex < md)
            o->print(s, 1, 1, false);
        }
    else
        {
        o->clear_line(1);
        o->draw_grid(1);
        s = o->welcomeText.substr(o->welcomeIndex - md - 20, md);
        o->print(s, 1, 1, false);
        }
    o->welcomeIndex++;
    if (o->welcomeIndex >= (int)o->welcomeText.length()+20)
        {
        gtk_timeout_remove(o->welcomeTag);
        }
    return 1;
    }


int MustuxLcd::blink_icon(gpointer obj)
    {
    MustuxLcd *o = (MustuxLcd*) obj;
    if (o->isIconBlinking)
            {
            o->clear();
            o->draw_background_grid();
            o->draw_icon(o->iconToBlink);
            }
    else
            {
            o->clear();
            o->draw_background_grid();
            }
    o->isIconBlinking = !o->isIconBlinking;
    return 1;
    }


// eof

