#ifndef LCD_HH
#define LCD_HH

#include <string>
#include <gtk/gtk.h>
#include <gdk/gdk.h>

#include "MustuxPane.hh"
class MustuxComponent;

class MustuxLcd : public MustuxPane
        {
        public :
            MustuxLcd(MustuxComponent* parent);
            ~MustuxLcd();

            int build( int width, int height, bool isResizeable);
            int init();
            int finalize();
            int unbuild();
            void recreate();
            void rect (int x, int y , int w , int h, bool fill);
            void fast_clear(int x, int y, int w, int h);
            void fast_rect (int x, int y, int w, int h, bool fill);
            void draw_string(string s, int x, int y);
            void draw_background_grid();
            void draw_icon(int icon);
            void clear_line(int line);
            void draw_grid(int line);
            void draw_all_grids();
            void draw_digit(int index, int col,  int line);
            void print(string s, int line);
            void print(string s, int col, int line);
            void print(string s, int line, bool clean);
            void print(string s, int col, int line, bool clean);
            void clear();
            void say_welcome(string text);
            int get_max_digits() const;
            string welcomeText;

            static const int DEFAULT_WIDTH = 100;
            static const int DEFAULT_HEIGHT = 50;

        protected :
            int maxDigits;
            int iconToBlink;
            int iconBlinkTag;
            bool isIconBlinking;
            bool isGridLcd;
            int welcomeIndex;
            int welcomeTag;

            // LCD
            static const int DOTSIZE = 2;
            static const int DOTHSPACE = 3;
            static const int DOTVSPACE = 3;
            static const int DIGITHSPACE = 12;
            static const int DIGITVSPACE = 17;
            static const int GRID_MARGIN_LEFT = 8;
            static const int GRID_MARGIN_UP = 10;

            // ICONS
            static const int PLAY_ICON = 0;
            static const int PAUSE_ICON = 1;
            static const int REC_ICON = 2;
            static const int RENDER_ICON = 3;

            int start_blinking_icon(int icon);
            int stop_blinking_icon();

            // callbacks
            static int blink_icon(gpointer o);
            static int scroll_welcome(gpointer o);
        };

#include "MustuxComponent.hh"

#endif

//eof
