#include "MustuxNoteBook.hh"
#include "MustuxDebugger.hh"

MustuxNoteBook::MustuxNoteBook(MustuxComponent* parent) : MustuxComponent(parent)
        {
        PENTERCONS;
        PEXITCONS;
        }

MustuxNoteBook::~MustuxNoteBook()
        {
        PENTERDES;
        PEXITDES;
        }


int MustuxNoteBook::build()
        {
        PENTER;
        build(MustuxComponent::HORIZONTAL);
        return 1;
        PEXIT;
        }

int MustuxNoteBook::build(int direction)
        {
        PENTER;
        fillDirection = direction;
        PMESG("Building a MustuxNoteBook " << (fillDirection == MustuxComponent::HORIZONTAL?"horizontally":"vertically") << " fillable");
        wProjectsContainer = gtk_notebook_new();

        gtk_notebook_set_tab_pos (GTK_NOTEBOOK(wProjectsContainer), GTK_POS_TOP);
        gtk_notebook_set_scrollable ( GTK_NOTEBOOK(wProjectsContainer), true);
        gtk_widget_show (wProjectsContainer);

        gtk_container_add (GTK_CONTAINER(parent->get_box()), wProjectsContainer);

        PEXIT;
        return 1;
        }


int MustuxNoteBook::init()
        {
        PENTER;
        MustuxComponent::init();
        currProject=-1;
        for (int i=0; i<MAX_PROJECTS; i++)
                project[i]=0;
        gtk_signal_connect (GTK_OBJECT (wProjectsContainer), "switch_page",GTK_SIGNAL_FUNC (on_switch_project), this);
        gtk_signal_connect (GTK_OBJECT (wProjectsContainer), "configure_event",GTK_SIGNAL_FUNC (on_configure_projects_area), this);
        PENTER;
        return 1;
        }


int MustuxNoteBook::finalize()
        {
        PENTER;
        MustuxComponent::finalize();
        PEXIT;
        return 1;
        }


int MustuxNoteBook::unbuild()
        {
        PENTER;
        MustuxComponent::unbuild();
        PEXIT;
        return 1;
        }


int MustuxNoteBook::add_project()
        {
        if (fillDirection == MustuxComponent::HORIZONTAL)
                {
                //PMESG("Building Horizontal Box " << (isHomogeneous?"":"Not") << " Homogeneous");
                project[++currProject] = gtk_hbox_new (false, 0);
                }
        else
                {
                //PMESG("Building Vertical Box " << (isHomogeneous?"":"Not") << " Homogeneous");
                project[++currProject] = gtk_vbox_new (false, 0);
                }
        gtk_container_add (GTK_CONTAINER (wProjectsContainer), project[currProject]);
        gtk_widget_show (project[currProject]);
        return currProject;
        }



int MustuxNoteBook::get_curr_project_number()
        {
        return currProject;
        }

GtkWidget* MustuxNoteBook::get_current_project_container()
        {
        return project[currProject];
        }

void MustuxNoteBook::set_label(int projNumber, string slabel)
        {
        GtkWidget *label;
        label = gtk_label_new (slabel.c_str());
        gtk_widget_show (label);
        gtk_notebook_set_tab_label (GTK_NOTEBOOK (wProjectsContainer),
                                    project[projNumber], label);
        }


void MustuxNoteBook::set_project(int p)
        {
        currProject=p;
        gtk_notebook_set_page (GTK_NOTEBOOK(wProjectsContainer),p);
        }


// callbacks

int MustuxNoteBook::on_switch_project(GtkNotebook *w, GtkNotebookPage *p, int pageNum, MustuxNoteBook* o)
    {
    return 1;
    }


int MustuxNoteBook::on_configure_projects_area(GtkWidget *w, GdkEventConfigure *e, MustuxNoteBook* o)
    {
    return 1;
    }


// eof

