#include <iostream>
#include <string>
#include "MustuxDebugger.hh"
#include "MustuxColorManager.hh"
#include "MustuxPane.hh"

MustuxPane::MustuxPane(MustuxWindow* parent) : MustuxComponent(parent)
        {
        PENTERCONS;
        PEXITCONS;
        }

MustuxPane::MustuxPane(MustuxComponent* parent) : MustuxComponent(parent)
        {
        PENTERCONS;
        PEXITCONS;
        }

MustuxPane::MustuxPane(GtkWidget* aParentBox) : MustuxComponent(aParentBox)
        {
        PENTERCONS;
        PEXITCONS;;
        }

MustuxPane::~MustuxPane()
    {
    PENTERDES;
    PEXITDES;
    }


int MustuxPane::build()
    {
    PENTER;
    build(DEFAULT_WIDTH, DEFAULT_HEIGHT, MustuxComponent::RESIZEABLE, MustuxComponent::FILLABLE);
    PENTER;
    return 1;
    }


int MustuxPane::build(int width, int height)
    {
    PENTER;
    build(width, height, MustuxComponent::RESIZEABLE, MustuxComponent::FILLABLE);
    PEXIT;
    return 1;
    }


int MustuxPane::build(int width, int height, bool isResizeable)
    {
    PENTER;
    build(width,height, isResizeable, MustuxComponent::FILLABLE);
    PEXIT;
    return 1;
    }


int MustuxPane::build(int width, int height, bool isResizeable, bool isFillable)
        {
        PENTER;
        MustuxComponent::build(MustuxComponent::HORIZONTAL, MustuxComponent::NOT_HOMOGENEOUS, isResizeable, isFillable);
        drawArea = gtk_drawing_area_new ();
        gtk_widget_set_events ( drawArea,
                                GDK_EXPOSURE_MASK
                                | GDK_LEAVE_NOTIFY_MASK
                                | GDK_BUTTON_PRESS_MASK
                                | GDK_POINTER_MOTION_MASK
                                | GDK_POINTER_MOTION_HINT_MASK );
        gtk_drawing_area_size (GTK_DRAWING_AREA(drawArea), width, height);
        gtk_widget_show (drawArea);
        gtk_box_pack_end (GTK_BOX (box), drawArea, isResizeable, isFillable, 0);

        drawPane = drawArea->window;
        storePane = gdk_pixmap_new(drawPane, width, height,-1);
        standardFont = gdk_font_load("-schumacher-clean-medium-r-normal-*-*-120-*-*-c-*-iso646.1991-irv");
        PEXIT;
        return 1;
        }


int MustuxPane::init()
    {
    PENTER;
    gtk_signal_connect (GTK_OBJECT (drawArea), "expose_event", GTK_SIGNAL_FUNC (on_expose), this);
    gtk_signal_connect (GTK_OBJECT (drawArea), "configure_event", GTK_SIGNAL_FUNC (on_reconfigure), this);
    clear();
    PEXIT;
    return 1;
    }

int MustuxPane::finalize()
    {
    MustuxComponent::finalize();
    return 1;
    }

int MustuxPane::unbuild()
    {
    MustuxComponent::unbuild();
    return 1;
    }


void MustuxPane::update()
        {
        PFLOODENTER;
        update(0, 0, -1, -1);
        PFLOODEXIT;
        }


void MustuxPane::update(int x, int y, int width, int height)
        {
        PFLOODENTER;
        gdk_draw_pixmap(drawPane ,
                        drawArea->style->white_gc, storePane, x, y, x, y,
                        width, height);
        PFLOODEXIT;
        }


void MustuxPane::print(string s, int line)
        {
        }



void MustuxPane::recreate() // virtual
    {
    PENTER;
    clear(MustuxColorManager::BLACK);
    PEXIT;
    }


int MustuxPane::get_background_color()
    {
    return MustuxColorManager::BLACK;
    }


void MustuxPane::reconfigure()
    {
    PFLOODENTER;
    if (storePane)
            gdk_pixmap_unref(storePane);
    storePane = gdk_pixmap_new(drawPane, get_width(), get_height(), -1);
    recreate();
    PFLOODEXIT;
    }


void MustuxPane::clear() // virtual
    {
    PFLOODENTER;
    PFLOODEXIT;
    }


void MustuxPane::clear(int color)
    {
    PFLOODENTER;
    gdk_draw_rectangle(storePane, MustuxColorManager::get_color(color) , true, 0, 0, get_width(), get_height());
    update(0,0,-1,-1);
    PFLOODEXIT;
    }

void MustuxPane::draw_pixel(int x, int y)
    {
    PFLOODENTER;
    gdk_draw_point(storePane, MustuxColorManager::get_color(MustuxColorManager::BLACK), x, y);
    update(x-1, y-1, 2, 2);
    PFLOODEXIT;
    }

void MustuxPane::draw_line(int x, int y, int x2, int y2)
    {
    PFLOODENTER;
    gdk_draw_line(storePane, MustuxColorManager::get_color(MustuxColorManager::BLACK), x, y, x2, y2);
    update(0,0,-1,-1); // Optimize me , please!
    PFLOODEXIT;
    }


void MustuxPane::draw_line(int x, int y, int x2, int y2, int color)
    {
    PFLOODENTER;
    gdk_draw_line(storePane, MustuxColorManager::get_color(color), x, y, x2, y2);
    update(0,0,-1,-1);  // Optimize me , please!
    PFLOODEXIT;
    }


void MustuxPane::draw_string(string s, int x, int y)
    {
    PFLOODENTER;
    const char* cs = s.c_str();
    gdk_draw_string(storePane, standardFont,
                    MustuxColorManager::get_color(MustuxColorManager::BLACK),
                    x , y , cs);
    update(0,0,-1,-1); // Optimize me , please!
    PFLOODEXIT;
    }


void MustuxPane::draw_rectangle( int x, int y , int w , int h, bool fill, int color)
    {
    PFLOODENTER;
    gdk_draw_rectangle(storePane, MustuxColorManager::get_color(color), fill, x , y, w, h);
    update(x-1,y-1,w+2,h+2);
    PFLOODEXIT;
    }


int MustuxPane::get_width()
    {
    return drawArea->allocation.width;
    }


int MustuxPane::get_height()
    {
    return drawArea->allocation.height;
    }

void MustuxPane::activate()
    {
    PFLOODENTER;
    draw_rectangle(0,0,get_width()-1,get_height()-1,false,MustuxColorManager::RED);
    PFLOODEXIT;
    }


void MustuxPane::deactivate()
    {
    PFLOODENTER;
    draw_rectangle(0,0,get_width()-1,get_height()-1,false,get_background_color());
    PFLOODEXIT;
    }


bool MustuxPane::is_pointed()
    {
    double x, y ,p, xt, yt;
    GdkModifierType mask	;
    gdk_input_window_get_pointer (drawPane, GDK_CORE_POINTER , &x, &y, &p , &xt, &yt, &mask);
    return ((y>=0) && (y<=get_height())) && ((x>=0) && (x<=get_width()));
    }


GtkWidget* MustuxPane::get_draw_area() 
    {
    return drawArea;
    }


GdkDrawable* MustuxPane::get_draw_pane()
    {
    return drawPane;
    }

GdkPixmap* MustuxPane::get_store_pane()
    {
    return storePane;
    }

int MustuxPane::get_mouse_x()
    {
    double x, y ,p, xt, yt;
    GdkModifierType mask;
    gdk_input_window_get_pointer (drawPane, GDK_CORE_POINTER , &x, &y, &p , &xt, &yt, &mask);
    return (int) x;
    }


int MustuxPane::get_mouse_y()
    {
    double x, y ,p, xt, yt;
    GdkModifierType mask;
    gdk_input_window_get_pointer (drawPane, GDK_CORE_POINTER , &x, &y, &p , &xt, &yt, &mask);
    return (int) y;
    }


// callbaks

int MustuxPane::on_expose(GtkWidget *widget, GdkEventExpose *event, MustuxPane *o)
    {
    o->update();
    return 1;
    }


int MustuxPane::on_reconfigure(GtkWidget *widget, GdkEventConfigure *event, MustuxPane *o)
    {
    o->reconfigure();
    return 1;
    }

