#ifndef MUSTUXPANE_HH
#define MUSTUXPANE_HH

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include "MustuxComponent.hh"

class MustuxWindow;


class MustuxPane : public MustuxComponent
    {
    public :
            MustuxPane(MustuxWindow *parentComponent);
            MustuxPane(MustuxComponent *parentComponent);
            MustuxPane(GtkWidget* aParentBox);
            ~MustuxPane();

            int build();
            int build(int width, int height);
            int build(int width, int height, bool isHomogeneous);
            int build(int width, int height, bool isHomogeneous, bool isFillable);
            int init();
            int unbuild();
            int finalize();
            virtual void clear();
            void clear(int color);
            void draw_pixel(int x, int y);
            void draw_line(int x, int y, int x2, int y2);
            void draw_line(int x, int y, int x2, int y2, int color);
            void draw_rectangle( int x, int y , int w , int h, bool fill, int color);
            void draw_string(string message, int x, int y);
            virtual void print(string s, int line);
            virtual int get_background_color();
            int get_width();
            int get_height();
            void activate();
            void deactivate();
            bool is_pointed();
            GtkWidget* get_draw_area();
            GdkDrawable* get_draw_pane();
            GdkPixmap* get_store_pane();

            static const int DEFAULT_WIDTH = 50;
            static const int DEFAULT_HEIGHT = 50;
    protected :
            GtkWidget *drawArea;
            GdkDrawable *drawPane;
            GdkPixmap *storePane;
            GdkFont *standardFont;
            GdkBitmap *alphamask;

            bool scalable;

            void update();
            void update(int x, int y, int width, int height);
            void reconfigure();
            virtual void recreate();

            int get_mouse_x();
            int get_mouse_y();


            // callbacks
            static int on_expose(GtkWidget *widget,
                            GdkEventExpose *event,
                            MustuxPane *o);

            static int on_reconfigure(GtkWidget *widget,
                            GdkEventConfigure *event,
                            MustuxPane *o);
    };

#include "MustuxWindow.hh"

#endif

