#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <string>
#include "MustuxProgressBar.hh"
#include "MustuxDebugger.hh"
#include "MustuxColorManager.hh"


MustuxProgressBar::MustuxProgressBar():MustuxWindow()
        {
        PENTERCONS;
        PEXITCONS;
        }


MustuxProgressBar::~MustuxProgressBar()
        {
        PENTERDES;
        finalize();
        unbuild();
        PEXITDES;
        }


int MustuxProgressBar::build()
        {
        PENTER;
        MustuxWindow::build(300,100, MustuxComponent::RESIZEABLE);
        wLabel = gtk_label_new ("Progress");
        gtk_widget_show (wLabel);
        gtk_box_pack_start (GTK_BOX (box), wLabel, false, false, 0);

        barContainer = new MustuxComponent(this);
        barContainer->build(MustuxComponent::VERTICAL,
                         MustuxComponent::NOT_HOMOGENEOUS,
                         MustuxComponent::RESIZEABLE,
                         MustuxComponent::FILLABLE);

        bar = new MustuxPane(barContainer);
        bar->build(300,20);

        PEXIT;
        return 1;
        }

int MustuxProgressBar::init(string title)
        {
        PENTER;
        MustuxWindow::init();
        set_title(title);
        set_label(title);
        barContainer->init();
        bar->init();
        bar->clear(MustuxColorManager::WHITE);
        bar->draw_rectangle(0, 0, bar->get_width(), bar->get_height(), false, MustuxColorManager::BLACK);
        PEXIT;
        return 1;
        }


int MustuxProgressBar::finalize()
        {
        PENTER;
        MustuxWindow::finalize();
        PEXIT;
        return 1;
        }


int MustuxProgressBar::unbuild()
        {
        PENTER;
        MustuxWindow::unbuild();
        PEXIT;
        return 1;
        }


void MustuxProgressBar::set_label(string label)
        {
        PENTER;
        gtk_label_set_text(GTK_LABEL(wLabel), label.c_str());
        PEXIT;
        }


void MustuxProgressBar::configure(float aMin, float aMax, float aInit)
        {
        min = aMin;
        max = aMax;
        curr = aInit;
        }



void MustuxProgressBar::set_value(float v)
        {
        //PMESG("Setting Progress Bar value to " << v);
        lastCurr = curr;
        curr = v;
        update();
        }


void MustuxProgressBar::update()
        {
        //PMESG("drawing bar : current value = " << curr);
        bar->draw_rectangle( (int) ((lastCurr/max) * bar->get_width() ), 1, bar->get_width()-1, bar->get_height()-1, true, MustuxColorManager::WHITE);
        bar->draw_rectangle(1, 1, (int) ((curr/max) * bar->get_width() ) -1, bar->get_height()-1, true, MustuxColorManager::BLACK);
        flush_events();
        }


//eof


