#include "MustuxDebugger.hh"
#include "MustuxWindow.hh"
#include "MustuxComponent.hh"


// constructors and general methods

MustuxWindow::MustuxWindow()
    {
    PENTERCONS;
    PEXITCONS;
    }

MustuxWindow::~MustuxWindow()
    {
    PENTERDES;
    PEXITDES;
    }

int MustuxWindow::build()
    {
    build(MustuxComponent::VERTICAL,MustuxComponent::NOT_HOMOGENEOUS, MustuxComponent::RESIZEABLE);
    return 1;
    }


int MustuxWindow::build(int aFillDirection)
    {
    build(aFillDirection, MustuxComponent::NOT_HOMOGENEOUS, MustuxComponent::RESIZEABLE);
    return 1;
    }


int MustuxWindow::build(int aFillDirection, bool isHomogeneous )
    {
    build(aFillDirection, isHomogeneous, MustuxComponent::RESIZEABLE);
    return 1;
    }


int MustuxWindow::build(int aFillDirection, bool isHomogeneous, bool isResizeable)
    {
    PENTER;
    fillDirection = aFillDirection;
    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    set_resizeable(isResizeable);
    gtk_widget_set_events (window ,
                           GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK
                         | GDK_EXPOSURE_MASK  | GDK_POINTER_MOTION_MASK
                         | GDK_POINTER_MOTION_HINT_MASK);
    gtk_widget_show(window);
    if (fillDirection == MustuxComponent::HORIZONTAL)
        {
        PMESG("Building Horizontal Box " << (isHomogeneous?"":"Not") << " Homogeneous");
        box = gtk_hbox_new (isHomogeneous, 0);

        }
    else
        {
        PMESG("Building Vertical Box " << (isHomogeneous?"":"Not") << " Homogeneous");
        box = gtk_vbox_new (isHomogeneous, 0);
        }
    gtk_widget_show (box);
    gtk_container_add (GTK_CONTAINER (window), box);
    //standardFont = 	gdk_font_load ("-dec-terminal-medium-r-normal-*-*-140-*-*-c-*-iso8859-1");
    gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
    PEXIT;
    return 1;
    }



int MustuxWindow::init()
    {
    PENTER;
    //gtk_signal_connect (GTK_OBJECT (window), "delete_event", GTK_SIGNAL_FUNC (on_delete), this);
    PEXIT;
    return 1;
    }


int MustuxWindow::finalize()
    {
    PENTER;
    PEXIT;
    return 1;
    }


int MustuxWindow::unbuild()
    {
    PENTER;
    gtk_widget_hide(window);
    PEXIT;
    return 1;
    }


void MustuxWindow::center()
	{
	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
    }



void MustuxWindow::set_resizeable(bool resizeable)
	{
    if (resizeable)
		gtk_window_set_policy(GTK_WINDOW(window), false,true,false);
    else
		gtk_window_set_policy(GTK_WINDOW(window), false,false,true);
	}


void MustuxWindow::set_title(string title)
    {
    PENTER;
    gtk_window_set_title (GTK_WINDOW (window), title.c_str());
    PEXIT;
    }


void MustuxWindow::set_fill_direction(int newFillDirection)
    {
    fillDirection = newFillDirection;
    }


void MustuxWindow::add_separator()
    {
    PENTER;
    if (fillDirection == MustuxComponent::HORIZONTAL)
        {
        GtkWidget* hseparator = gtk_hseparator_new ();
        gtk_widget_show (hseparator);
        gtk_box_pack_start(GTK_BOX (box), hseparator, false, false, 3);
        }
    else
        {
        GtkWidget* vseparator = gtk_vseparator_new ();
        gtk_widget_show (vseparator);
        gtk_box_pack_start(GTK_BOX (box), vseparator, false, false, 3);
        }
    PEXIT;
    }

GtkWidget* MustuxWindow::get_window()
    {
    PENTER;
    PEXIT;
    return window;
    }


GtkWidget* MustuxWindow::get_box()
    {
    PENTER;
    PEXIT;
    return box;
    }


void MustuxWindow::start_interaction()
    {
    PENTER;
    gtk_main();
    PEXIT;
    }
    
void MustuxWindow::stop_interaction()
    {
    PENTER;
    gtk_main_quit();
    PEXIT;
    }

void MustuxWindow::flush_events()
        {
        while (gtk_events_pending())
                gtk_main_iteration();
        }

// callbacks

int MustuxWindow::on_delete(GtkWidget* w, MustuxWindow* o)
    {
    o->unbuild();
    return 1;
    }


// eof

