#ifndef PROTUXWINDOW_HH
#define PROTUXWINDOW_HH

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <string>

/** A MustuxWindow is a base class for other component such MustuxDialog and MustuxInterface. 
 *  A MustuxWindow is itself a component container and can one or more MustuxComponent objects in it.
 */
class MustuxWindow
        {
        public:
                MustuxWindow();
                ~MustuxWindow();

                /** Build a basic window, that is also a component which can contain other components .
                 *  The components within the window follow a vertical non-homogeneous distribution
                 */
                int build();


                /** Build a basic window, that is also a component which can contain other components .
                 *  The components within the window follow a non-homogeneous distribution.
                 *  @param fillDirection : can be MUSTUXCOMPONENT::VERTICAL or MUSTUXCOMPONENT::HORIZONTAL
                 */
                int build(int fillDirection);


                /** Build a basic window, that is also a component which can contain other components .
                 *  @param fillDirection : can be MUSTUXCOMPONENT::VERTICAL or MUSTUXCOMPONENT::HORIZONTAL
                 *  @param isHomogeneous : true makes subcomponents be arranged homogeneously
                 */
                int build(int fillDirection, bool isHomogeneous);


                /** Build a basic window, that is also a component which can contain other components .
                 *  @param fillDirection : can be MUSTUXCOMPONENT::VERTICAL or MUSTUXCOMPONENT::HORIZONTAL
                 *  @param isHomogeneous : true makes subcomponents be arranged homogeneously
                 *  @param isResizeable : true makes subcomponents be resizeable
                 */
                int build(int fillDirection, bool isHomogeneous, bool isResizeable);


                /** 
                 * Shows the window, initializes internal properties and prepare event handling engines
                 */
                int init();

                
                /** Starts event handling and lock the window to catch events within it
                 */
                void start_interaction();
                
                
                /** Stop event handling.
                 */
                void stop_interaction();


                /** 
                 * Hide the window and de-activate internal event handling 
                 */
                int finalize();

                /** 
                 * Deallocate memory used for the window 
                 */
                int unbuild();


                /** 
                 * sets windows to be reziseable or not
                 */
                void set_resizeable(bool isResizeable);


                /** 
                 * sets the director to arrange sub components
                 */
                void set_fill_direction(int newFillDirection);

                
                /** 
                 * centers the window in the screen 
                 */
                void center();


                /** 
                 * changes the title of the window
                 */                
                void set_title(string title);


                /** 
                 * Returns the GTK widget associated to window 
                 */
                GtkWidget* get_window();


                /** 
                 * Returns the internal GTK box used to hold sub-components 
                 */
                GtkWidget* get_box();


                /** append a separator as a subcomponent. It will be a horizonal or a
                 *  vertical separator according to the fill direction
                 */
                void add_separator();


                void flush_events();
                


        protected:
                GtkWidget *window;
                GtkWidget *box;
                GdkFont *standardFont;

                int fillDirection;
                static int on_delete(GtkWidget* w, MustuxWindow* o);

        };



#endif


//

