#include <cstdio>
#include <iostream>
#include <cmath>
#include "MustuxLib/MustuxLib.hh"
#include "Audio.hh"
#include "Peak.hh"

// FOR LATER :#include "GlobalSettings.hh"


Peak::Peak()
        {
        PENTERCONS;
        PEXITCONS;
        }


Peak::~Peak()
        {
        PENTERDES;
        PEXITDES;
        }



int Peak::build(Audio* anAudioSource)
        {
        PENTER;
        audioSource = anAudioSource;
        audioSourceFile = audioSource->get_file();
        audioSourceFileSize = audioSource->get_file_size();
        audioSize = audioSource->get_audio_size();
        headerSize = audioSource->get_header_size();
        blockSize = audioSource->get_block_size();
        channels = audioSource->get_channels();
        rate = audioSource->get_rate();
        bitDepth = audioSource->get_bit_depth();

        for (int hzoom = 0; hzoom<MAX_ZOOM_LEVELS; hzoom++)
                {
                peakStep[hzoom] = (long) pow(2,hzoom);
                RAMBuffer[hzoom] = NULL;
                RAMBufferSize[hzoom]=0;
                }

        MustuxProgressBar* pb = new MustuxProgressBar();
        pb->build();
        pb->init("Build Peak Buffers...");
        //pb->flush_events();
        pb->configure(0.0 , 100.0 , 0.0);

        long long tempstep;
        long long pos;
        for (int hzoom=MAX_ZOOM_USING_SOURCEFILE+1; hzoom<MAX_ZOOM_LEVELS; hzoom++)
        {
                tempstep = peakStep[hzoom] * blockSize;
                RAMBufferSize[hzoom] =  ((audioSize / tempstep)+1) * blockSize;
                RAMBuffer[hzoom] = new char[RAMBufferSize[hzoom]];
                //PMESG("Creating RAM buffer for HZOOM = " << hzoom <<" of " << RAMBufferSize[hzoom] << " bytes");
                int bs = blockSize;
                char c;
                pos = headerSize;
                int i=0;
                do
                        {
                        fseek(audioSourceFile, pos, SEEK_SET);
                        for (int k=0; k<bs; k++)
                                {
                                fread(&c, 1, 1, audioSourceFile);
                                RAMBuffer[hzoom][i++]=c;
                                }
                        pos += tempstep;
                        }
                while (pos<audioSourceFileSize);
                float f =  (float) hzoom / MAX_ZOOM_LEVELS;
                pb->set_value(f*100.0);
                pb->update();
                }
        delete pb;
        PEXIT;
        return 1;
        }



int Peak::unbuild()
        {
fclose(audioSourceFile);
        for (int i=0; i<MAX_ZOOM_LEVELS; i++)
                if (RAMBuffer[i])
                        delete RAMBuffer[i];
        return 1;
        }



int Peak::fill_buffer(char* bufferToFill, long long startBlock, int numBlocksToFill,  int hzoom)
        {
        PFLOODENTER;
        int blocksFilled;
        if (hzoom > MAX_ZOOM_USING_SOURCEFILE)
                {
                char* pi = RAMBuffer[hzoom]+((startBlock / peakStep[hzoom]) * blockSize);
                char* pf = RAMBuffer[hzoom]+RAMBufferSize[hzoom];
                int totalBytes = numBlocksToFill * blockSize;
                if (totalBytes >= (pf-pi))
                totalBytes = pf-pi;
                memcpy(bufferToFill, pi, totalBytes);
                blocksFilled = totalBytes / blockSize;
                }
        else if (hzoom <= MAX_ZOOM_USING_SOURCEFILE)
                {
                int step;
                long long pos;
                step = blockSize * peakStep[hzoom];
                pos = startBlock * blockSize + headerSize;
                long long originalPos = ftell(audioSourceFile);
                blocksFilled = 0;
                char c;
                char* p = bufferToFill;
                do
                        {
                        fseek(audioSourceFile, pos, SEEK_SET);
                        for (int k=0; k < blockSize; k++)
                                {
                                fread(&c, 1, 1, audioSourceFile);
                                *(p++)=c;
                                }
                        blocksFilled++;
                        pos+=step;
                        }
                while ((blocksFilled < numBlocksToFill) && (!feof(audioSourceFile)));
                fseek(audioSourceFile, originalPos, SEEK_SET);
                }
        PFLOODEXIT;
        return blocksFilled;
        }






void Peak::show_info()
        {
        PMESG("Peakfiles information\n");
        for (int i=0; i<MAX_ZOOM_LEVELS; i++)
                {
                PMESG("Zoom 1:" << (int)pow(2,i) << "\t\t - "  <<
                (i>MAX_ZOOM_USING_SOURCEFILE?"Using RAM buffer":"Using Source File") <<
                "Peak step=" << peakStep[i]);
                }
        }

long Peak::get_peak_step(int hzoom) const
        {
        return peakStep[hzoom];
        }



// eof
