#ifndef PEAK_HH
#define PEAK_HH

class Audio;

class Peak
	{
	public:
		Peak();
		~Peak();
		
		/** Instantiate a Peak Object and generate RAM buffers containing the peak images for 
		 *  zoom levels equal or greater than 6. Zoom levels between 1 and 5 uses "real-peak" 
		 *  images (peaks which are read direct from the audio file);
		 */
		int build(Audio* audiosource);

		/** Deallocate memory resources allocated by build() */
		int unbuild();

		/** Receives an empty buffer and fills it with peaks, according to the zoom level passed. */
		int fill_buffer(char* bufferToFill, long long startBlock, int numBlocksToFill , int hzoom);

		/** Display information about the RAM buffers on the console */
		void show_info();

		/** Returns the peak step (number of blocks between two peaks in a same channel) for a given hzoom factor*/
		long get_peak_step(int hzoom) const;
		
  	  static const int MAX_ZOOM_LEVELS = 15;
        static const int MAX_ZOOM_USING_SOURCEFILE = 6;
        
    private :
		Audio* audioSource;
		FILE* audioSourceFile;
		long long audioSourceFileSize;
		long long audioSize;

		/** number of BLOCKS between two shown peaks */
		long peakStep[MAX_ZOOM_LEVELS]; 
    	char* RAMBuffer[MAX_ZOOM_LEVELS];
		long long RAMBufferSize[MAX_ZOOM_LEVELS];

		int headerSize;
		int blockSize;
		int channels;
		int rate;
		int bitDepth;
	};

#include "Audio.hh"

#endif
	
	


//eof

