#include "PrafFile.hh"
#include "MustuxLib/MustuxDebugger.hh"

#define BUFFER_SIZE 1024


PrafFile::PrafFile()
    {
    PENTERCONS;
    PEXITCONS;
    }


PrafFile::~PrafFile()
    {
    PENTERDES;
    PEXITDES;
    }


int PrafFile::open(string newFileName)
    {
    PENTER;
    fileName=newFileName;
    file = fopen(fileName.c_str(),"r");
    if (!file)
        {
        PEXIT;
        return -1;
        }
    PEXIT;
    return 1;
    }


int PrafFile::create(string newFileName)
    {
    PENTER;
    fileName=newFileName;
    file = fopen(fileName.c_str(),"w+");
    if (!file)
        {
        PEXIT;
        return -1;
        }
    PEXIT;
    return 1;
    }

int PrafFile::read_header()
    {
    PENTER;
    rewind();
    fread(&(formatLabel), 4, 1 , file);
    if ((formatLabel[0]!='P') ||
        (formatLabel[1]!='R') ||
        (formatLabel[2]!='A') ||
        (formatLabel[3]!='F'))
            {
            PERROR("This seems not to be a PRAF file.");
            return -1;
            }
    fread(&(channels), 4, 1 , file);
    fread(&(rate), 4, 1 , file);
    fread(&(bitDepth) , 4, 1 , file);
    fread(&(audioSize) , 8, 1 , file);
    fread(&(projectLabel), PRAF_HEADER_LABEL_SIZE, 1 , file);
    fread(&(groupLabel), PRAF_HEADER_LABEL_SIZE, 1 , file);
    fread(&(hardwareLabel), PRAF_HEADER_LABEL_SIZE, 1 , file);
    fread(&(description), PRAF_HEADER_DESCRIPTION_SIZE, 1 , file);
    fread(&(reserved), PRAF_HEADER_RESERVED_SIZE,1, file);
    headerSize=PRAF_HEADER_SIZE;
    PEXIT;
    return 1;
    }


int PrafFile::write_header(int newChannels, int newRate, int newBitDepth, long long newAudioSize,
                             string newProjectLabel, string newGroupLabel, string newHardwareLabel,
                             string newDescription)
        {
        PENTER;
        formatLabel[0] = 'P';
        formatLabel[1] = 'R';
        formatLabel[2] = 'A';
        formatLabel[3] = 'F';
        for (int i=0; i<PRAF_HEADER_LABEL_SIZE; i++) projectLabel[i] = 0;
        for (int i=0; i<PRAF_HEADER_LABEL_SIZE; i++) groupLabel[i] = 0;
        for (int i=0; i<PRAF_HEADER_LABEL_SIZE; i++) hardwareLabel[i] = 0;
        for (int i=0; i<PRAF_HEADER_DESCRIPTION_SIZE; i++) description[i] = 0;
        for (int i=0; i<PRAF_HEADER_RESERVED_SIZE; i++) reserved[i]=0;
        for (int i=0; i < (int) newProjectLabel.length(); i++) projectLabel[i] = newProjectLabel.c_str()[i];
        for (int i=0; i < (int) newGroupLabel.length(); i++) groupLabel[i] = newGroupLabel.c_str()[i];
        for (int i=0; i < (int) newHardwareLabel.length(); i++) hardwareLabel[i] = newHardwareLabel.c_str()[i];
        for (int i=0; i < (int) newDescription.length(); i++) description[i] = newDescription.c_str()[i];
        for (int i=0; i<PRAF_HEADER_RESERVED_SIZE; i++) reserved[i] = 0;
        channels = newChannels;
        rate = newRate;
        bitDepth = newBitDepth;
        audioSize = newAudioSize;
        headerSize=PRAF_HEADER_SIZE;
        rewind();
        fwrite(&(formatLabel), 4, 1 , file);
        fwrite(&(channels), 4, 1 , file);
        fwrite(&(rate), 4, 1 , file);
        fwrite(&(bitDepth) , 4, 1 , file);
        fwrite(&(audioSize) , 8, 1 , file);
        fwrite(&(projectLabel), PRAF_HEADER_LABEL_SIZE, 1 , file);
        fwrite(&(groupLabel), PRAF_HEADER_LABEL_SIZE, 1 , file);
        fwrite(&(hardwareLabel), PRAF_HEADER_LABEL_SIZE, 1 , file);
        fwrite(&(description), PRAF_HEADER_DESCRIPTION_SIZE, 1 , file);
        fwrite(&(reserved), PRAF_HEADER_RESERVED_SIZE, 1, file);
        PEXIT;
        }



int PrafFile::close_file()
        {
        PENTER;
        int r = fclose(file);
        PEXIT;
        return r;
        }



int PrafFile::append_samples_from_wav(WavFile* wavFile)
        {
        PENTER;
        char buf[BUFFER_SIZE];
        size_t size;

        wavFile->rewind();
        wavFile->skip_header();
        skip_header();

        int i=0;
        cout << endl;
        FILE* input = wavFile->get_file();
        while (!feof(input))
            {
            size = fread (&buf, 1, BUFFER_SIZE, input);
            fwrite(&buf, 1 , size, file);
            cout << "read " << ++i << " buffers" << "     \r";
            }
        cout<<endl<<endl;
        PEXIT;
        return 1;
        }



int PrafFile::append_samples_from_buffer(char* buffer, int bufferSize)
        {
        PFLOODENTER;
        size_t size ;
        size = fwrite(buffer, 1 , bufferSize, file);
        int r = 1;
        if ((int) size < bufferSize)
                {
                PERROR("Could not append the whole buffer to file");
                r = -1;
                }
        PFLOODEXIT;
        return r;
        }


void PrafFile::show_info()
        {
        PENTER;
        PMESG("File       : " << fileName);
        PMESG("Channels   : " << (channels == 1 ? "Mono" : "Stereo"));
        PMESG("Rate       : " << rate << " Hz");
        PMESG("bitDepth   : " << bitDepth << " bits");
        //PMESG("Project : " << projectLabel);
        //PMESG("Group : " + groupLabel);
        //PMESG("Hardware : " + hardwareLabel);
        //PMESG("Description : " + description);*/
        PEXIT;
        }

void PrafFile::rewind()
        {
        std::rewind(file);
        }


void PrafFile::skip_header()
        {
        fseek(file, headerSize, SEEK_SET);
        }


int PrafFile::get_channels()
        {
        return channels;
        }

int PrafFile::get_rate()
        {
        return rate;
        }

int PrafFile::get_bit_depth()
        {
        return bitDepth;
        }


long long PrafFile::get_audio_size()
        {
        return audioSize;
        }



long long PrafFile::get_file_size()
        {
	fseek(file, 0, SEEK_END);
        return ftell(file);
        }


int PrafFile::get_header_size()
        {
        return headerSize;
        }





string PrafFile::get_project_label()
        {
        PENTER;
        char cs[PRAF_HEADER_LABEL_SIZE+1];
        strncpy(cs, projectLabel,PRAF_HEADER_LABEL_SIZE); cs[PRAF_HEADER_LABEL_SIZE]=0;
        PEXIT;
        return string(cs);
        }


string PrafFile::get_group_label()
        {
        PENTER;
        char cs[PRAF_HEADER_LABEL_SIZE+1];
        strncpy(cs, groupLabel,PRAF_HEADER_LABEL_SIZE); cs[PRAF_HEADER_LABEL_SIZE]=0;
        PEXIT;
        return string(cs);
        }


string PrafFile::get_hardware_label()
        {
        PENTER;
        char cs[PRAF_HEADER_LABEL_SIZE+1];
        strncpy(cs, hardwareLabel,PRAF_HEADER_LABEL_SIZE); cs[PRAF_HEADER_LABEL_SIZE]=0;
        PEXIT;
        return string(cs);
        }


string PrafFile::get_description()
        {
        PENTER;
        char cs[PRAF_HEADER_DESCRIPTION_SIZE+1];
        strncpy(cs, description,PRAF_HEADER_DESCRIPTION_SIZE); cs[PRAF_HEADER_DESCRIPTION_SIZE]=0;
        PEXIT;
        return string(cs);
        }


FILE* PrafFile::get_file()
        {
        return file;
        }

string PrafFile::get_filename()
        {
        return fileName;
        }

int PrafFile::fix_audio_size()
        {
        PENTER;
        fseek(file, 0, SEEK_END);
        long long fileSize = ftell(file);
        audioSize = fileSize - get_header_size();
        fseek(file, 16, SEEK_SET);
        fwrite(&audioSize, 1, 8, file);
        return 1;
        }

// eof
