#ifndef PRAFFILE_HH
#define PRAFFILE_HH

// Must merge with PrafHeader class
#include <string>
#include <cstdio>

class WavFile;

class PrafFile
    {
    public:
        static const int PRAF_HEADER_LABEL_SIZE = 16;
        static const int PRAF_HEADER_DESCRIPTION_SIZE = 56;
        static const int PRAF_HEADER_SIZE = 256;
        static const int PRAF_HEADER_RESERVED_SIZE = 128;

        PrafFile();
        ~PrafFile();
        int open(string newFileName);
        int create(string newFileName);
        int read_header();
        int write_header(int newChannels, int newRate, int newBitDepth, long long newAudioSize,
                         string newProjectLabel, string newGroupLabel, string newHardwareLabel,
                         string newDescription);
        int append_samples_from_wav(WavFile* wavfile);
        int append_samples_from_buffer(char* buffer, int bufferSize);
        void show_info();
        void rewind();
        void skip_header();
        string get_project_label();
        string get_group_label();
        string get_hardware_label();
        string get_description();
        int get_channels();
        int get_rate();
        int get_bit_depth();
        long long get_audio_size();
        int get_header_size();
        FILE* get_file();
        string get_filename();
        long long get_file_size();
        int fix_audio_size();

        int close_file();


    private:

        FILE* file;
        string fileName;
        char formatLabel[4];
        int channels;
        int rate;
        int bitDepth;
        long long audioSize;
        int headerSize;
        char projectLabel[PRAF_HEADER_LABEL_SIZE];
        char groupLabel[PRAF_HEADER_LABEL_SIZE];
        char hardwareLabel[PRAF_HEADER_LABEL_SIZE];
        char description[PRAF_HEADER_DESCRIPTION_SIZE];
        char reserved[PRAF_HEADER_RESERVED_SIZE];
        };

#include "WavFile.hh"


#endif

