#include <string>

//#include <iostream>
//#include <strstream>
//#include <cmath>

#include "MustuxLib/MustuxLib.hh"
#include "Track.hh"
#include "MultiTrackArea.hh"

Track::Track(MultiTrackArea* aParentMTA, int aBaseX, int aBaseY, int aWidth, int aHeight)
        {
        PENTERCONS;
        parentMTA = aParentMTA;
        baseX = aBaseX;
        baseY = aBaseY;
        width = aWidth;
        height = aHeight;
        clipList=0;
        PEXITCONS;
        }


Track::~Track()
        {
        PENTERDES;
        PEXITDES;
        }


void Track::recreate()
        {
        PFLOODENTER;
        clear();
        draw();
        PFLOODEXIT;
        }

void Track::clear()
        {
        PFLOODENTER;
        parentMTA->draw_rectangle(baseX, baseY, width, height, true, MustuxColorManager::TRACK_BACKGROUND_COLOR);
        parentMTA->draw_line(baseX, baseY + height, width, baseY + height, MustuxColorManager::BLACK);
        parentMTA->draw_line(baseX, baseY + height -1 , width, baseY + height -1, MustuxColorManager::WHITE);
        PFLOODEXIT;
        }

void Track::activate()
        {
        PFLOODENTER;
        isActive=true;
        for (int i=0; i<3; i++)
                parentMTA->draw_rectangle( baseX+i , baseY + i , width - i*2 , height - i*2 - 2 , false, MustuxColorManager::RED);
        PFLOODEXIT;
        }


void Track::deactivate()
        {
        PFLOODENTER;
        isActive=false;
        for (int i=0; i<4; i++)
                parentMTA->draw_rectangle( baseX+i , baseY + i , width - i*2 , height - i*2 - 2 , false, MustuxColorManager::TRACK_BACKGROUND_COLOR);
        PFLOODEXIT;
        }


bool Track::is_pointed()
        {
        double x, y ,p, xt, yt;
        GdkModifierType mask	;
        gdk_input_window_get_pointer (parentMTA->get_draw_pane(), GDK_CORE_POINTER , &x, &y, &p , &xt, &yt, &mask);
        return ((x>=baseX) && (x<=baseX+width) && (y>=baseY) && (y<=baseY+height));
        }


int Track::fill(long long insertBlock, Audio* audio)
        {
        PENTER;
        PMESG("Creating clip starting in " << insertBlock);
        add_clip(audio, insertBlock, 0, audio->get_total_blocks());
        recreate();
        PEXIT;
        return 1;
        }



int Track::split_clip(long long splitPoint)
        {
        PENTER;
        PMESG("Trying to find clip under " << splitPoint);
        AudioClip* c = get_clip_under(splitPoint);
        if (c!=0)
                {
                PMESG("I am upon clip " << c);
                long long leftLenght = splitPoint - (c->trackFirstBlock);
                long long rightLenght = c->lenght - leftLenght;
                long long rightSourceFirstBlock = c->sourceFirstBlock + leftLenght;
                c->sourceLastBlock = c->sourceFirstBlock + leftLenght;
                c->trackLastBlock = c->trackFirstBlock + leftLenght;
                c->lenght=leftLenght;
                add_clip(c->get_audio_source(), splitPoint, rightSourceFirstBlock, rightLenght);
                }
        else
                {
                PMESG("I am upon no clip :-(");
                }
        PEXIT;
        return 0;
        }



int Track::draw()
    {
    PFLOODENTER;
    AudioClip* c=clipList;
    while (c)
    	{
        c->draw();
        c=c->next;
        }
    parentMTA->update();
    PFLOODEXIT;
    return 1;
    }


void Track::draw_axis()
        {
        PFLOODENTER;
        parentMTA->draw_line(baseX, baseY + height/3, baseX + width, baseY + height/3);
        parentMTA->draw_line(baseX, baseY + 2* height/3, baseX + width, baseY + 2* height/3);
        PFLOODEXIT;
        }



AudioClip* Track::get_clip_under(long long blockPos)
    {
    AudioClip* c=clipList;
    while (c)
        {
        long long fb = c->trackFirstBlock;
        long long lb = c->trackLastBlock;

        if ((fb<=blockPos) && (lb>=blockPos))
            {
            //PMESG("Checked clip : SFB=" << fb <<" , SLB=" << lb << " . Found!!");
            return c;
            }
        else
            {
            //PMESG("Checked clip : SFB=" << fb <<" , SLB=" << lb);
            }
        c=c->next;
        }
    return 0;
    }



AudioClip* Track::get_next_clip_after(long long blockPos)
        {
        AudioClip* c=clipList;
        while (c)
                {
                if (c->trackFirstBlock > blockPos)
                        {
                        return c;
                        }
                c=c->next;
                }
        return 0;
        }



int Track::delete_clip(AudioClip* clip)
        {
        PENTER;
        if (!clip)
                {
                PERROR("Trying to delete invalid Clip");
                PEXIT;
                return -1;
                }
        if (clip->prev==0) // if it is the first clip...
                {
                PMESG("deleting first clip!");
                clipList=clip->next;
                if (clipList!=0)
                        clipList->prev=0;
                delete clip;
                }
        else
        if (clip->next==0) // if it is the last clip...
                {
                PMESG("deleting last clip!");
                clip->prev->next=0;
                delete clip;
                }
        else
                {
                PMESG("deleting clip...");
                clip->prev->next = clip->next;
                clip->next->prev = clip->prev;
                delete clip;
                }
        PEXIT;
        return 1;
        }


int Track::add_clip(Audio* audioSource, long long trackInsertPos, long long sourceFirstBlock, long long lenght)
        {
        PENTER;
        AudioClip *clip = new AudioClip(this, audioSource, sourceFirstBlock, lenght, trackInsertPos);
        AudioClip* c;
        AudioClip* c2;

        if (!clipList)
                {
                PMESG("Adding first clip :  [ TFB = " << clip->trackFirstBlock <<
                                        " , TLB = " << clip->trackLastBlock <<
                                        " , SFB = " << clip->sourceFirstBlock <<
                                        " , SLB = " << clip->sourceLastBlock <<
                                        " , LEN = " << clip->lenght << " ]");
                clipList = clip;
                clip->prev=0;
                clip->next=0;
                }
        else
                {
                c=clipList;
                c2=clipList;
                while (c && c->trackFirstBlock < trackInsertPos)
                        {
                        c2=c;
                        c=c->next;
                        }
                if (!c)
                        {
                        PMESG("Adding at the end of list");
                        c2->next=clip;
                        clip->prev=c2;
                        clip->next=0;
                        }
                else if (c==clipList)
                        {
                        PMESG("Adding at the begin of list");
                        clip->next=clipList;
                        clipList->prev=clip;
                        clip->prev=0;
                        clipList=clip;
                        }
                else
                        {
                        PMESG("Adding in the middle of list");
                        clip->next=c2->next;
                        clip->prev=c2;
                        c2->next=clip;
                        }
                }
        PMESG("Entire clip list now is ");
        c=clipList;
        int i=0;
        while (c)
                {
                PMESG("Clip #" << i++
                << " : SFB=" << c->sourceFirstBlock
                << " , SLB=" <<c->sourceLastBlock
                << " , TRB="<<c->trackFirstBlock);
                c=c->next;
                }
        PEXIT;
        return 1;
        }






// eof
