#include "VUMeter.hh"
#include "Lcd.hh"
#include <cmath>

#define LEVEL_DOTS 40
#define XPOS 15
#define YPOS 70
#define DOT_HEIGHT 10
#define HSPACE 3
#define CLIPXPOS XPOS+LEVEL_DOTS*2+3

const float MAXLOG = log(32768);
const float MINLOG = log(1);
const float LOGRANGE = MAXLOG - MINLOG;

static int _reconfigure(GtkWidget *widget, GdkEventConfigure *event, VUMeter* o)
	{
	o->reconfigure();
	return 1;
	}
	
	
void VUMeter::start()
	{
	PENTER(" (VU Meter) ID = " << this->ID);
	init();
	recreate();
	gtk_signal_connect (GTK_OBJECT (drawarea), "configure_event", GTK_SIGNAL_FUNC (_reconfigure), this);	
	active = false;
	updating = false;
	old_sampleL = 32400;
	old_sampleR = 32400;	
	sampleL = 0;
	sampleR = 0;
	start_updating();	
	PEXIT("");	
	}

void VUMeter::recreate()
	{
	PENTER(" (VU Meter)");
	clear();
	draw_grids();	
	update();
	PEXIT("");
	}


void VUMeter::reconfigure()
	{
	PENTER(" (VU Meter)");
	if (pixmap)gdk_pixmap_unref(pixmap);
	pixmap = gdk_pixmap_new(drawarea->window, drawarea->allocation.width,
 				   drawarea->allocation.height, -1);
	recreate();
	PEXIT("");
	}


void VUMeter::draw_grids()
	{
	PENTER("");
	draw_string("L", XPOS-13, YPOS+8);
	draw_string("R", XPOS-13, YPOS+DOT_HEIGHT+HSPACE+8);	
	for (int i = 0; i<LEVEL_DOTS; i++)	
		{
		gdk_draw_rectangle(pixmap, Colors::get_color(LCD_OFF_DOT_COLOR),
					true, XPOS+i*2, YPOS, 1, DOT_HEIGHT);
		gdk_draw_rectangle(pixmap, Colors::get_color(LCD_OFF_DOT_COLOR),
					true, XPOS+i*2, YPOS+DOT_HEIGHT+HSPACE, 1, DOT_HEIGHT);
		}	
	PEXIT("");
	}

	
void VUMeter::set_audio_source(Audio* as)
	{
	PENTER("");	
	audio_source = as;
	PEXIT("");	
	}


static int _refresh_VU(gpointer obj)
	{
	VUMeter* o = (VUMeter*) obj;	
	o->refresh_VU();
	return 1;
	}

void VUMeter::activate()
	{
	PENTER("");	
	active = true;
	clear();
	draw_grids();
	start_updating();
	PEXIT("");	
	}


void VUMeter::deactivate()
	{
	PENTER("");	
	active = false;
	PEXIT("");	
	}


void VUMeter::start_updating()
	{
	PENTER("");
	updating = true;
	update_tag = gtk_timeout_add( 20, _refresh_VU, this );	
	PEXIT("");
	}


void VUMeter::stop_updating()
	{
	PENTER("");
	static GdkGC* c2 = Colors::get_color(LCD_OFF_DOT_COLOR);
	old_sampleL = 0;
	old_sampleR = 0;
	sampleL = 0;
	sampleR = 0;
	updating = false;
	for (int i = 0; i< LEVEL_DOTS; i++)
		gdk_draw_rectangle(drawarea->window, c2, true, XPOS+i*2, YPOS, 1, DOT_HEIGHT);
	for (int i = 0; i< LEVEL_DOTS; i++)
		gdk_draw_rectangle(drawarea->window, c2, true, XPOS+i*2, YPOS+DOT_HEIGHT+HSPACE, 1, DOT_HEIGHT);
	gtk_timeout_remove(update_tag);
	PEXIT("");
	}


void VUMeter::refresh_VU()
	{
	static int yL;
	static int yR;
	static GdkGC* c1 = Colors::get_color(LCD_ON_DOT_COLOR);
	static GdkGC* c2 = Colors::get_color(LCD_OFF_DOT_COLOR);
	static GdkGC* c3 = Colors::get_color(LCD_CRITICAL_DOT_COLOR);
	if (active && (audio_source->get_dsp_status()==STOPPED))
		deactivate();
	if (active)
		{
		short* audio_buff = (short*) audio_source->get_current_audio_fragment();
		short s1 = abs(audio_buff[0])+1;
		short s2 = abs(audio_buff[1])+1;
		sampleL = (s1 > old_sampleL ? s1 : old_sampleL - 500);
		sampleR = (s2 > old_sampleR ? s2 : old_sampleR - 500);
		}
	else
		{
		sampleL = old_sampleL - 500;
		sampleR = old_sampleR - 500;
		}

	if (!active && (sampleL<1) && (sampleR<1))
		{
		stop_updating();
		return;
		}

        // nova formula e' essa. Substituir !
        double dbL = 20 * log (sampleL/32768);
        double dbR = 20 * log (sampleR/32768);

	double dL = log(sampleL);
	double dR = log(sampleR);
//	printf("%f , %f \n", dL,dR);
	float fL = (dL-MINLOG)/LOGRANGE;//((float) sampleL / 32768) ;
	float fR = (dR-MINLOG)/LOGRANGE;//((float) sampleR / 32768);
	if ( fL >= 0.99)
		gdk_draw_rectangle(drawarea->window, c3, true, CLIPXPOS, YPOS, 20, DOT_HEIGHT);
	if ( fR >= 0.99)
		gdk_draw_rectangle(drawarea->window, c3, true, CLIPXPOS, YPOS+DOT_HEIGHT+HSPACE, 20, DOT_HEIGHT);
	yL = (short) (fL * LEVEL_DOTS);
	yR = (short) (fR * LEVEL_DOTS);

	// LEFT VU
	if (yL >= 0)
		{
		for (int i = 0; i< yL; i++)
			gdk_draw_rectangle(drawarea->window, c1, true, XPOS+i*2, YPOS, 1, DOT_HEIGHT);
		for (int i = yL; i< LEVEL_DOTS; i++)
			gdk_draw_rectangle(drawarea->window, c2, true, XPOS+i*2, YPOS, 1, DOT_HEIGHT);
		}
		
  	// RIGHT VU
	if (yR >= 0)
	       {
		for (int i = 0; i< yR; i++)
			gdk_draw_rectangle(drawarea->window, c1, true, XPOS+i*2, YPOS+DOT_HEIGHT+HSPACE, 1, DOT_HEIGHT);
		for (int i = yR; i< LEVEL_DOTS; i++)
			gdk_draw_rectangle(drawarea->window, c2, true, XPOS+i*2, YPOS+DOT_HEIGHT+HSPACE, 1, DOT_HEIGHT);	
		}
	
	old_sampleL = sampleL;
	old_sampleR = sampleR;
	}



// eof
